/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: KelpSample.java,v 1.1.2.3 2000/03/23 06:57:27 pmahar Exp $
 */

package org.enhydra.kelp.sample;

import java.io.*;
import java.math.BigDecimal;
import java.util.Enumeration;
import com.lutris.util.*;
import com.lutris.logging.Logger;
import com.lutris.logging.LogChannel;
import com.lutris.appserver.server.LBS;
import com.lutris.appserver.server.session.*;
import com.lutris.appserver.server.user.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.httpPresentation.*;


/**
 * Application class for this specific application.  This implements
 * standard startup and shutdown methods and well as application specific
 * ones.
 */
public class KelpSample extends StandardApplication {
  /*
   * This is the global Config object that is shared by the
   * whole application. If there was an error reading the file, this
   * will be null.
   *
   * @see Config
   */
  public static Config config = null;

  private final String CONFIG_GREETING  = "Greeting";
  private final String CONFIG_APP_ROOT  = "AppRoot";
  private final String DEFAULT_GREETING = "Aloha";

  /**
   * Get the greeting text from the application configuration file.
   */
  public String getGreeting() {
    String greeting = DEFAULT_GREETING;
    try {
      greeting = config.getString(CONFIG_GREETING);
    } catch (com.lutris.util.ConfigException e) {
      greeting = DEFAULT_GREETING;
    }
    return greeting;
  }

  /**
   * Get the greeting text from the application configuration file.
   */
  public String getAppRoot() {
    // For Ennydra 3.0 you can also better calculate the default root by
    // calling: config.getConfigFile().getFile().getParentFile().getParent()

    String defRoot = "../../../../";
    String appRoot = defRoot;
    try {
      appRoot = config.getString(CONFIG_APP_ROOT);
    } catch (com.lutris.util.ConfigException e) {
      appRoot = defRoot;
    }
    return appRoot;
  }

  /**
   * Start the application.
   *
   * @param appConfig Configuration object.
   * @exception ApplicationException if an error occurs starting the
   *  application.
   */
  public void startup(Config appConfig) throws ApplicationException {
    /*
     * This will create the config and sessionManager objects.
     */
    super.startup(appConfig);
    if (logChannel != null) {
        logChannel.write(Logger.INFO,
                         "Starting up the Enhydra Sample for JBuilder");
    }
    config = appConfig;
  }

  /**
   * Shutdown the application.
   */
  public void shutdown() {
    super.shutdown();
  }

  public boolean requestPreprocessor(HttpPresentationComms comms)
          throws Exception
  {
    super.requestPreprocessor(comms);
    if (comms.session == null) {
        // Session not setup; so its not a presentation object.
        return false;
    }

    String pp = comms.request.getApplicationPath();
    if (!pp.endsWith("/")) {
        pp += "/";
    }
    String loginDir = pp;
    String mediaDir = pp;
    String page = comms.request.getRequestURI();
    if (page.startsWith(loginDir) || page.startsWith(mediaDir)) {
        /*
         *   Let them go to the login and logout areas whether or not
         *   they are logged in.
         */
        return false;
    } else {
        /*
         *   All other pages require the user to be logged in.
         */
        if (comms.session.getUser() != null)
            // It's ok, they are logged in already.
            return false;
        else {
            // They are not logged in! Redirect to login page.
            String url = pp + "Greeting.po";
            throw new ClientPageRedirectException(url);
        }
    }
  }
}

