/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: WizardPage.java,v 1.3 2000/02/10 19:55:57 pmahar Exp $
 */

package org.enhydra.devsys.wizard.common;

import org.enhydra.devsys.swing.GridBagConstraintsX;
import java.awt.*;
import java.beans.*;
import javax.swing.*;

/**
 * A WizardPage is a panel that you can add to a WizardDialog. It displays
 * a page title, instructions and a WizardPanel.
 *
 * @author
 * Paul Mahar
 * @version %I%, %G%
 */
public class WizardPage extends JPanel {
    private BorderLayout  layoutMain;
    private JLabel	  image;
    private JPanel	  panelWizard;
    private GridBagLayout layoutWizard;
    private JLabel	  pageTitle;
    private JTextArea     instructions;
    private JPanel	  panelCustom;
    private BorderLayout  layoutCustom;

    /**
     * Create a WizardPage.
     */
    public WizardPage() {
	try {
	    jbInit();
	    pmInit();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Get the instructions that display in a normal font under the page title.
     */
    public String getInstructions() {
	return instructions.getText();
    }

    /**
     * Set the instructions that display in a normal font under the page title.
     * The instructions will word wrap.
     *
     * @param s
     * A paragraph to display in a read only word wrapped text area
     */
    public void setInstructions(String s) {
	instructions.setText(s);
    }

    /**
     * Get the text that appears in bold at the top of the page.
     */
    public String getPageTitle() {
	return pageTitle.getText();
    }

    /**
     * Set the text that appears in bold at the top of the page.
     */
    public void setPageTitle(String s) {
	pageTitle.setText(s);
    }

    /**
     * Overrides super.add()
     * <P>
     * Add a WizadPanel under the instructions.
     * </P>
     * @param panel
     * The panel containing swing controls for gathering information
     * specific to this wizard page.
     */
    public void add(WizardPanel panel) {
	panelCustom.add(panel, BorderLayout.CENTER);
    }

    /**
     * Do construtor operations that are not generated by
     * the JBuilder designer.
     */
    private void pmInit() {
	image.setText("");
    }

    /**
     * UI code generated by JBuilder.
     * <P><I>Do not modify the signature of this method.</I></P>
     */
    private void jbInit() throws Exception {
	panelWizard = (JPanel) Beans.instantiate(getClass().getClassLoader(),
						 JPanel.class.getName());
	layoutWizard =
	    (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
					      GridBagLayout.class.getName());
	pageTitle = (JLabel) Beans.instantiate(getClass().getClassLoader(),
					       JLabel.class.getName());
	instructions =
	    (JTextArea) Beans.instantiate(getClass().getClassLoader(),
					  JTextArea.class.getName());
	panelCustom = (JPanel) Beans.instantiate(getClass().getClassLoader(),
						 JPanel.class.getName());
	layoutCustom =
	    (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
					     BorderLayout.class.getName());
	layoutMain =
	    (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
					     BorderLayout.class.getName());
	image = (JLabel) Beans.instantiate(getClass().getClassLoader(),
					   JLabel.class.getName());

	this.setLayout(layoutMain);
	panelWizard.setLayout(layoutWizard);
	pageTitle.setFont(new java.awt.Font("Dialog", 1, 12));
	pageTitle.setText("Page Title");
	instructions.setLineWrap(true);
	instructions.setWrapStyleWord(true);
	instructions.setDisabledTextColor(SystemColor.windowText);
	instructions.setBackground(SystemColor.inactiveCaptionBorder);
	instructions.setEnabled(false);
	instructions.setText("Instructions");
	instructions.setEditable(false);
	instructions.setFont(new java.awt.Font("Dialog", 0, 12));
	panelCustom.setLayout(layoutCustom);
	this.add(image, BorderLayout.WEST);
	this.add(panelWizard, BorderLayout.CENTER);
	panelWizard.add(pageTitle, new GridBagConstraintsX(0, 0, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 5, 2, 5), 5, 5));
	panelWizard.add(instructions, new GridBagConstraintsX(0, 1, 1, 1, 0.2, 0.2
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 5, 2, 5), 5, 5));
	panelWizard.add(panelCustom, new GridBagConstraintsX(0, 2, 1, 2, 0.7, 0.7
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 5, 2, 5), 5, 5));
    }

}



/*--- formatting done in "Sun Java Convention" style on 01-16-2000 ---*/

