/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: NewAppPanel.java,v 1.3 2000/02/10 19:55:57 pmahar Exp $
 */

package org.enhydra.devsys.wizard.newapp;

import org.enhydra.devsys.wizard.common.WizardPanel;
import org.enhydra.devsys.swing.SwingUtil;
import org.enhydra.devsys.swing.GridBagConstraintsX;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.beans.*;

/**
 * The NewAppPanel defines the panel for entering the paramerters required
 * to create a new Enhydra application with LutrisAppTree.
 *
 * @author
 * Ray Kiuchi
 * @version %I%, %G%
 */
public class NewAppPanel extends WizardPanel implements NewAppProperties {
    private GridBagLayout       layoutMain = new GridBagLayout();
    private JTextField		appTextField = new JTextField();
    private JTextField		rootTextField = new JTextField();
    private JTextField		packageTextField = new JTextField();
    private JTextField		enhydraTextField = new JTextField();
    private String		appName = "Sample";
    private String		rootDir;
    private String		packageName = "sample";
    private String		enhydraDir = "/usr/local/enhydra";
    private JButton		buttonRoot;
    private JButton		buttonEnhydra;
    private LocalButtonListener buttonListener;

    private JPanel panelDirectory;
    private GridBagLayout layoutDirectory;
    private TitledBorder borderDirectory;
    private JLabel labelEnhydra;
    private JLabel labelRoot;

    private JPanel panelApp;
    private GridBagLayout layoutApp;
    private TitledBorder borderApp;
    private JLabel labelApp;
    private JLabel labelPackage;


    /**
     * Create a panel to place in NewAppPage.
     */
    public NewAppPanel() {
	try {
	    jbInit();
	    pmInit();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Get the name to use when generating an Enhydra app.
     * <P><I>
     * NewAppProperties implementation
     * </I></P>
     */
    public String getAppName() {
	return appTextField.getText();
    }

    /**
     * Set the name to use when generating an application.
     * <P><I>
     * NewAppProperties implementation
     * </I></P>
     */
    public void setAppName(String text) {
	appTextField.setText(text);
    }

    /**
     * Get the root directory where to create the application directory.
     * <P><I>
     * NewAppProperties implementation
     * </I></P>
     */
    public String getRootDir() {
	return rootTextField.getText();
    }

    /**
     * Set the root directory where to create the application directory.
     * <P><I>
     * NewAppProperties implementation
     * </I></P>
     */
    public void setRootDir(String text) {
	rootTextField.setText(text);
    }

    /**
     * Get the package name to use when generating an application.
     * <P><I>
     * NewAppProperties implementation
     * </I></P>
     */
    public String getPackageName() {
	return packageTextField.getText();
    }

    /**
     * Set the package name to use when generating an application.
     * <P><I>
     * NewAppProperties implementation
     * </I></P>
     */
    public void setPackageName(String text) {
	packageTextField.setText(text);
    }

    /**
     * Get the Enhydra home directory. The default is /usr/local/enhydra.
     * <P><I>
     * NewAppProperties implementation
     * </I></P>
     */
    public String getEnhydraDir() {
	return enhydraTextField.getText();
    }

    /**
     * Set the Enhydra home directory. The default is /usr/local/enhydra.
     * <P><I>
     * NewAppProperties implementation
     * </I></P>
     */
    public void setEnhydraDir(String text) {
	enhydraTextField.setText(text);
    }

    /**
     * Do construtor operations that are not generated by
     * the JBuilder designer.
     */
    private void pmInit() {
	buttonListener = new LocalButtonListener();

	buttonRoot.addActionListener(buttonListener);
	buttonEnhydra.addActionListener(buttonListener);
    }

    /**
     * UI code generated by JBuilder.
     * <P><I>Do not modify the signature of this method.</I></P>
     */
    private void jbInit() throws Exception {

	// this.setLargeIcon( new ImageIcon(ClassLoader.getSystemResource("com/lutris/devsys/newApp/media/AppWizard.gif")));
	buttonRoot = (JButton) Beans.instantiate(getClass().getClassLoader(),
						 JButton.class.getName());
	buttonEnhydra =
	    (JButton) Beans.instantiate(getClass().getClassLoader(),
					JButton.class.getName());

	panelDirectory   = (JPanel) Beans.instantiate(getClass().getClassLoader(), JPanel.class.getName());
	panelApp     = (JPanel) Beans.instantiate(getClass().getClassLoader(), JPanel.class.getName());

        layoutDirectory = (GridBagLayout) Beans.instantiate(getClass().getClassLoader(), GridBagLayout.class.getName());
        layoutApp     = (GridBagLayout) Beans.instantiate(getClass().getClassLoader(), GridBagLayout.class.getName());

        labelEnhydra = (JLabel) Beans.instantiate(getClass().getClassLoader(), JLabel.class.getName());
        labelRoot    = (JLabel) Beans.instantiate(getClass().getClassLoader(), JLabel.class.getName());
        labelApp     = (JLabel) Beans.instantiate(getClass().getClassLoader(), JLabel.class.getName());
        labelPackage = (JLabel) Beans.instantiate(getClass().getClassLoader(), JLabel.class.getName());

        appTextField.setToolTipText("Application name");
	appTextField.setText(appName);
	rootTextField.setPreferredSize(new Dimension(200, 21));
    rootTextField.setToolTipText("Application root directory");
	rootTextField.setText(rootDir);
	packageTextField.setToolTipText("Package name");
	packageTextField.setText(packageName);

	enhydraTextField.setPreferredSize(new Dimension(200, 21));
	enhydraTextField.setToolTipText("Enhydra home directory");
	enhydraTextField.setText(enhydraDir);

	buttonRoot.setText("Browse...");
        buttonRoot.setToolTipText(
          "Choose the directory where you want the application directory " +
          "to be placed");
	buttonEnhydra.setText("Browse...");
        buttonEnhydra.setToolTipText("Choose the Enhydra home directory:");

        labelEnhydra.setText("Enhydra home:");
        labelRoot.setToolTipText("");
    labelRoot.setText("Root to generate new application under:");
        labelApp.setToolTipText("");
    labelApp.setText("Name for new application directory to create:");
        labelPackage.setText("Package to use in generated classes:");

        borderDirectory = BorderFactory.createTitledBorder("Directories");
        panelDirectory.setBorder(borderDirectory);
        panelDirectory.setLayout(layoutDirectory);
        panelDirectory.add(labelEnhydra, new GridBagConstraintsX(0, 0, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(1, 5, 0, 5), 0, 0));
        panelDirectory.add(enhydraTextField, new GridBagConstraintsX(0, 1, 1, 1, 0.75, 0.75
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 4, 2), 0, 0));
	panelDirectory.add(buttonEnhydra, new GridBagConstraintsX(1, 0, 1, 2, 0.25, 0.25
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 2, 0, 2), 0, 0));
        panelDirectory.add(labelRoot, new GridBagConstraintsX(0, 2, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(4, 5, 0, 5), 0, 0));
	panelDirectory.add(rootTextField, new GridBagConstraintsX(0, 3, 1, 1, 0.75, 0.75
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 1, 2), 0, 0));
	panelDirectory.add(buttonRoot, new GridBagConstraintsX(1, 2, 1, 2, 0.25, 0.25
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 2, 0, 2), 0, 0));

        borderApp = BorderFactory.createTitledBorder("Application");
        panelApp.setBorder(borderApp);
        panelApp.setLayout(layoutApp);
        panelApp.add(labelApp, new GridBagConstraintsX(0, 0, 1, 1, 0.5, 0.5
            ,GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(1, 5, 0, 5), 0, 0));
	panelApp.add(appTextField, new GridBagConstraintsX(0, 1, 1, 1, 0.5, 0.5
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 4, 5), 0, 0));
        panelApp.add(labelPackage, new GridBagConstraintsX(0, 2, 1, 1, 0.5, 0.5
            ,GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(4, 5, 0, 5), 0, 0));
	panelApp.add(packageTextField, new GridBagConstraintsX(0, 3, 1, 1, 0.5, 0.5
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 1, 5), 0, 0));

	this.setLayout(layoutMain);

	this.setMinimumSize(new Dimension(150, 100));
	this.setPreferredSize(new Dimension(175, 124));

       this.add(panelDirectory, new GridBagConstraintsX(0, 0, 3, 1, 0.5, 0.5
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 5, 1, 5), 0, 0));
       this.add(panelApp, new GridBagConstraintsX(0, 1, 3, 1, 0.5, 0.5
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(1, 5, 0, 5), 0, 0));
    }

    /**
     * This inner class defines the actions for buttons in the panel.
     *
     * @author
     * Paul Mahar
     * @version %I%, %G%
     */
    private class LocalButtonListener implements ActionListener {

	/**
	 * ActionListener implementation.
	 *
	 * @param event
	 * Event that triggered this listener.
	 *
	 */
	public void actionPerformed(ActionEvent event) {
	    Object source = event.getSource();
	    File   choice;

	    if (source == buttonRoot) {
		choice =
		    SwingUtil.getDirectoryChoice(buttonRoot.getTopLevelAncestor(),
						 getRootDir(),
						 "Select the Source directory");

		if (choice != null) {
		    setRootDir(choice.toString());
		}
	    } else if (source == buttonEnhydra) {
		choice =
		    SwingUtil.getDirectoryChoice(buttonEnhydra.getTopLevelAncestor(),
						 getEnhydraDir(),
						 "Select Enhydra Directory");

		if (choice != null) {
		    setEnhydraDir(choice.toString());
		}
	    }
	}

    }

}



/*--- formatting done in "Sun Java Convention" style on 01-16-2000 ---*/

