/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * $Id: ProcessLauncher.java,v 1.2.2.1 2000/03/16 02:48:47 pmahar Exp $
 */
package org.enhydra.ide.jbuilder.common;

import org.enhydra.ide.jbuilder.common.capture.WriteEvent;
import org.enhydra.ide.jbuilder.common.capture.WriteListener;
import java.io.*;
import java.util.Vector;
import javax.swing.SwingUtilities;

/**
 * Class declaration
 * 
 * 
 * @author
 * @version %I%, %G%
 */
public class ProcessLauncher {
    private static Process    process;
    private ErrorStreamThread errorThread = null;
    private InputStreamThread inputThread = null;
    private BufferedWriter    outputBuffer = null;
    private String	      runDirectory = null;
    private String	      mainClass = new String();
    private String[]	      args = null;
    private Vector	      processListeners = new Vector();

    /**
     * Constructor declaration
     * 
     * 
     * @see
     */
    public ProcessLauncher() {}

    /**
     * Method declaration
     * 
     * 
     * @return
     * 
     * @see
     */
    public String getMainClass() {
	return mainClass;
    } 

    /**
     * Method declaration
     * 
     * 
     * @param main
     * 
     * @see
     */
    public void setMainClass(String main) {
	mainClass = main;
    } 

    /**
     * Method declaration
     * 
     * 
     * @return
     * 
     * @see
     */
    public String getRunDirectory() {
	return runDirectory;
    } 

    /**
     * Method declaration
     * 
     * 
     * @param dir
     * 
     * @see
     */
    public void setRunDirectory(String dir) {
	runDirectory = dir;
    } 

    /**
     * Method declaration
     * 
     * 
     * @return
     * 
     * @see
     */
    public String[] getArgs() {
	return args;
    } 

    /**
     * Method declaration
     * 
     * 
     * @param a
     * 
     * @see
     */
    public void setArgs(String[] a) {
	args = a;
    } 

    /**
     * Starts up and closes UI in seperate VM for testing.
     */
    public static void main(String[] args) {
	ProcessLauncher launcher;

	launcher = new ProcessLauncher();

	launcher.setMainClass("com.lutris.appserver.server.jolt.joltc.JoltCompile");
	launcher.setArgs(null);
	launcher.startProcess();

	try {
	    Thread.sleep(40000);
	} catch (java.lang.InterruptedException e) {
	    e.printStackTrace();
	} 

	launcher.stopProcess();
	System.out.println("done");    // NORES
	System.exit(0);
    } 

    /**
     * Closes VM for UI and associated threads.
     */
    public void stopProcess() {
	if (errorThread != null) {
	    errorThread.expire();
	} 

	if (inputThread != null) {
	    inputThread.expire();
	} 

	if (outputBuffer != null) {
	    try {
		outputBuffer.close();
	    } catch (java.io.IOException e) {}
	} 

	if (process != null) {
	    process.destroy();
	} 

	errorThread = null;
	inputThread = null;
	outputBuffer = null;
	process = null;
    } 

    /**
     * Method declaration
     * 
     * 
     * @return
     * 
     * @see
     */
    public boolean isProcessActive() {
	boolean active = false;

	if (process != null) {
	    try {
		process.exitValue();

		process = null;
	    } catch (IllegalThreadStateException e) {
		active = true;
	    } 
	} 

	return active;
    } 

    /**
     * Starts VM for UI and associated threads as needed.
     */
    public void startProcess() {
	String   classPath = new String();
	String   javaW = new String();
	Vector   command = new Vector();
	String[] cmdArray = null;
	Runtime  runtime = null;
	int      size = 0;

	// this creates a help viewer in a seperate VM
	// Check if help process is running.
	if (!isProcessActive()) {
	    classPath = System.getProperties().getProperty("java.class.path");
	    javaW = System.getProperties().getProperty("java.home") 
		    + File.separator + "bin" + File.separator + "javaw";

	    // this might only work on NT!!!
	    // if (getRunDirectory() != null) {
	    // commandBuf.append("cmd /C START /D" + getRunDirectory() + " ");
	    // }
	    command.addElement(javaW);
	    command.addElement("-Xms16m");
	    command.addElement("-Djava.compiler=symcjit");
	    command.addElement("-classpath");
	    command.addElement(quoteAsNeeded(classPath));
	    command.addElement(getMainClass());

	    if (getArgs() != null) {
		for (int i = 0; i < getArgs().length; i++) {
		    command.addElement(quoteAsNeeded(getArgs()[i]));
		} 
	    } 

	    try {
		runtime = Runtime.getRuntime();
		size = command.size();
		cmdArray = new String[size];

		for (int i = 0; i < size; i++) {
		    cmdArray[i] = (String) command.elementAt(i);
		} 

		process = runtime.exec(cmdArray);

		openOutputBuffer();
		startErrorThread();
		startInputThread();
	    } catch (java.io.IOException e) {
		e.printStackTrace();
	    } 
	} 
    } 

    // event methods

    /**
     * Method declaration
     * 
     * 
     * @param l
     * 
     * @see
     */
    public synchronized void addWriteListener(WriteListener l) {
	processListeners.addElement(l);
    } 

    /**
     * Method declaration
     * 
     * 
     * @param l
     * 
     * @see
     */
    public synchronized void removeWriteListener(WriteListener l) {
	if (processListeners.contains(l)) {
	    processListeners.removeElement(l);
	} 
    } 

    /**
     * Method declaration
     * 
     * 
     * @param e
     * 
     * @see
     */
    private void notifyWriteListeners(WriteEvent e) {
	int	      count = 0;
	WriteListener listener = null;

	count = processListeners.size();

	for (int i = 0; i < count; i++) {
	    listener = (WriteListener) processListeners.elementAt(i);

	    listener.onWrite(e);
	} 
    } 

    /**
     * Enclose string in quote arguments if it includes
     * enbedded spaces.
     */
    private String quoteAsNeeded(String in) {
	String out = new String();

	if (in != null) {
	    out = new String(in.trim());

	    if (out.indexOf(" ") > -1) {
		out = "\"" + out + "\"";
	    } 
	} 

	return out;
    } 

    /**
     * Method declaration
     * 
     * 
     * @see
     */
    private void startErrorThread() {
	if (errorThread != null) {
	    errorThread.expire();
	} 

	errorThread = new ErrorStreamThread();

	errorThread.start();
    } 

    /**
     * Method declaration
     * 
     * 
     * @see
     */
    private void startInputThread() {
	if (inputThread != null) {
	    inputThread.expire();
	} 

	inputThread = new InputStreamThread();

	inputThread.start();
    } 

    /**
     * Method declaration
     * 
     * 
     * @see
     */
    private void openOutputBuffer() {
	OutputStreamWriter writer;

	writer = new OutputStreamWriter(process.getOutputStream());
	outputBuffer = new BufferedWriter(writer);
    } 

    /**
     * Method declaration
     * 
     * 
     * @param type
     * @param message
     * 
     * @see
     */
    protected void notifyEventListeners(int type, String message) {
	notifyWriteListeners(new WriteEvent(this, type, message));
    } 

    /**
     * Class declaration
     * 
     * 
     * @author
     * @version %I%, %G%
     */
    class ErrorStreamThread extends InStreamThread {

	/**
	 * Constructor declaration
	 * 
	 * 
	 * @see
	 */
	public ErrorStreamThread() {
	    super();

	    setIn(process.getErrorStream());
	    setPrint(System.err);
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param message
	 * 
	 * @see
	 */
	protected void notifyListeners(String message) {
	    notifyEventListeners(WriteEvent.ERROR, message);
	} 

    }

    /**
     * Class declaration
     * 
     * 
     * @author
     * @version %I%, %G%
     */
    class InputStreamThread extends InStreamThread {

	/**
	 * Constructor declaration
	 * 
	 * 
	 * @see
	 */
	public InputStreamThread() {
	    super();

	    setIn(process.getInputStream());
	    setPrint(System.out);
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param message
	 * 
	 * @see
	 */
	protected void notifyListeners(String message) {
	    notifyEventListeners(WriteEvent.OUTPUT, message);
	} 

    }

    /**
     * Class declaration
     * 
     * 
     * @author
     * @version %I%, %G%
     */
    abstract class InStreamThread extends Thread {
	protected boolean	  fresh = true;
	private InputStream       in;
	private InputStreamReader reader;
	private BufferedReader    buffer;
	private PrintStream       print;

	/**
	 * Constructor declaration
	 * 
	 * 
	 * @see
	 */
	public InStreamThread() {
	    super();

	    setPriority(Thread.NORM_PRIORITY / 4);
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @see
	 */
	public void start() {
	    reader = new InputStreamReader(in);
	    buffer = new BufferedReader(reader);

	    super.start();
	} 

	/**
	 * Method declaration
	 * 
	 * 
	 * @see
	 */
	public void run() {
	    String lineIn = new String();

	    fresh = true;

	    while (fresh) {
		try {
		    if (buffer.ready()) {
			lineIn = buffer.readLine();

			if (print != null) {
			    print.println(lineIn);
			    notifyListeners(lineIn);
			} 
		    } else {
			sleep(1000);
		    } 
		} catch (java.lang.InterruptedException e) {
		    expire();
		} catch (java.io.IOException e) {
		    expire();
		} 
	    } 
	} 

	/**
	 * Method declaration
	 * 
	 * 
	 * @see
	 */
	public void expire() {
	    try {
		buffer.close();
		reader.close();
	    } catch (java.io.IOException e) {
		buffer = null;
		reader = null;
	    } 

	    fresh = false;

	    setPriority(Thread.MIN_PRIORITY);
	    interrupt();
	} 

	/**
	 * Method declaration
	 * 
	 * 
	 * @param newIn
	 * 
	 * @see
	 */
	public void setIn(InputStream newIn) {
	    in = newIn;
	} 

	/**
	 * Method declaration
	 * 
	 * 
	 * @param newPrint
	 * 
	 * @see
	 */
	public void setPrint(PrintStream newPrint) {
	    print = newPrint;
	} 

	/**
	 * Method declaration
	 * 
	 * 
	 * @param message
	 * 
	 * @see
	 */
	protected abstract void notifyListeners(String message);
    }

}



/*--- formatting done in "Sun Java Convention" style on 01-16-2000 ---*/

