/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: Mapper.java,v 1.1.2.2 2000/03/23 01:58:07 pmahar Exp $
 */

package org.enhydra.ide.jbuilder.common.node;

import java.io.File;

public class Mapper {
    private OtterProject project   = null;
    private String[][]   map       = new String[0][0];
    private String[]     path      = new String[0];
    private int          scope     = OtterProject.MAP_SCOPE_ALL;


    /**
     * Constructor declaration
     *
     *
     * @param p
     * @param f
     *
     * @see
     */
    public Mapper(OtterProject p) {
      project = p;
      scope = project.getMapScope();
      path  = project.getSourcePathArray();
      if (scope != OtterProject.MAP_SCOPE_NONE) {
         map = project.getPackageMap();
      }
    }

    public String getOutputImagePath(OtterImageFileNode image) {
      String imagePath = new String();
      String sourcePath = getSourcePathOf(image);
      String rawFile = (new File(image.getFilename())).getAbsolutePath();
      String  mappedPath = rawFile.substring(sourcePath.length());
      mappedPath = getMappedName(mappedPath, rawFile);
      mappedPath = mappedPath.replace('.',File.separatorChar);
      mappedPath = project.getOutputDirectory() +
         File.separator +
         mappedPath +
         rawFile.substring(rawFile.lastIndexOf("."));
      imagePath = mappedPath.replace('\\',File.separatorChar);
      imagePath = mappedPath.replace('/', File.separatorChar);
      return imagePath;
    }


    public String getClassName(OtterXMLCNode node)  {
        boolean custom     = false;
    	int     type       = node.getClassNameType();
    	String  sourcePath = getSourcePathOf(node);
    	String  rawFile    = (new File(node.getFilename())).getAbsolutePath();
    	String  mappedName =
    	   rawFile.substring(sourcePath.length(), rawFile.lastIndexOf("."));

    	switch (type) {
    	    case OtterXMLCNode.CLASS_NAME_CUSTOM:
    	       if (node.getCustomClassName().trim().length() > 0) {
                  mappedName = node.getCustomClassName();
                  custom = true;
                }
                break;
            case OtterXMLCNode.CLASS_NAME_DEFAULT:
                // already at default.
                break;
            default:
                 mappedName = getMappedName(mappedName, rawFile);
       }

       mappedName = validateClassName(mappedName, rawFile, type, custom);
       return mappedName;
     }

     public String validateClassName(String mappedName,
                                     String rawFile,
                                     int type,
                                     boolean custom) {
       String validName = mappedName.trim();
       validName = validName.replace('/','.');
       validName = validName.replace('\\','.');
       if (validName.startsWith(".")) {
          validName = validName.substring(1);
       }
      if (! custom) {
        int at = -1;
        if (validName.length()==0) {
           // not in path, create packageless default name.
           at = rawFile.lastIndexOf(File.separatorChar);
           validName = rawFile.substring(at + 1);
           at = validName.lastIndexOf(".");
           validName = validName.substring(0, at);
        }
        at = rawFile.lastIndexOf(File.separatorChar);
        String baseName = rawFile.substring(at + 1,
                          rawFile.lastIndexOf("."));
        at = validName.lastIndexOf(".");
        validName = validName.substring(0, at + 1)
                      + baseName + "HTML";
      }
      return validName;
    }

    private String getSourcePathOf(OtterNode node) {
      String[] path     = node.getProject().getSourcePathArray();
      String sourcePath = new String();
      File   pathDir    = null;
      File   nodeFile   = new File(node.getFilename());
      for (int i = 0 ; i < path.length ; i++ ) {
      	pathDir = new File(path[i]);
        if (nodeFile.getAbsolutePath().toLowerCase().startsWith(
                pathDir.getAbsolutePath().toLowerCase().trim())) {
          sourcePath = pathDir.getAbsolutePath().trim();
          break;
        }
      }
      return sourcePath;
    }

    private String getMappedName(String mappedName, String rawFile) {
        String   mappedPath = rawFile;
        String   matchPack  = new String();
        String[] lowerFrom  = null;
        String   lowerPath  = null;
        String   lowerFile  = null;
        String   toPackage  = new String();
        boolean  useMap = false;
        boolean  mapped = false;
        int      max    = -1;
        int      matchMax  = -1;

        lowerFile = rawFile.toLowerCase();

        if (map != null) {
            for (int i = 0; i < map.length; i++) {
                //  relative path support
                if (map[i][0].startsWith(".")) {
                  lowerFrom = new String[path.length];
                  for (int j = 0 ; j < lowerFrom.length ; j++ ) {
                     lowerFrom[j] = path[j].trim();
                     if (map[i][0].charAt(1) == File.separatorChar) {
                       lowerFrom[j] = lowerFrom[j] +
                            map[i][0].substring(1);
                     } else {
                       lowerFrom[j] = lowerFrom[j] +
                            File.separatorChar  +
                            map[i][0].substring(1);
                    }
                  }
                } else {
                  lowerFrom = new String[1];
                  lowerFrom[0] = map[i][0];
                }


                for (int j = 0 ; j < lowerFrom.length ; j++ ) {
                   lowerFrom[j] = lowerFrom[j].trim().toLowerCase();
                   if (lowerFrom[j].length() > max) {
                     max       = lowerFrom[j].length();
                     toPackage = map[i][1].trim();
                     if (lowerFile.length() > lowerFrom[j].length() &&
                        lowerFile.startsWith(lowerFrom[j])) {
                       matchPack = toPackage;
                       matchMax  = max;
                       mapped    = true;
                    }
                  }
                }
            }
        }

        int index = 0;
        if (mapped) {
            index = rawFile.lastIndexOf(".");
            mappedName = matchPack + rawFile.substring(matchMax, index);
        }
        return mappedName;
    }

}

