/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydrsa.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: OtterProject.java,v 1.7.2.7 2000/03/26 21:33:37 pmahar Exp $
 */

package org.enhydra.ide.jbuilder.common.node;

import org.enhydra.ide.jbuilder.common.PropUtil;
import java.io.File;
import java.util.Vector;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
abstract public class OtterProject implements OtterNode {
    private final String[][] DEFAULT_CONF_TABLE =
       {
          {" ../classes", " @OUTPUT@/../classes"},
          {"@OUTPUT@",    "./output"}
       };

    private final String XMLC_MAKE = "enhydra.xmlc.make";
    private final String XMLC_COMPILE_JAVA = "enhydra.xmlc.compileJava";
    private final String XMLC_KEEP_JAVA = "enhydra.xmlc.keepJava";
    private final String XMLC_DUMP = "enhydra.xmlc.dump";
    private final String XMLC_PARSEINFO = "enhydra.xmlc.parseInfo";
    private final String XMLC_PRINTINFO = "enhydra.xmlc.printInfo";
    private final String XMLC_VERBOSE = "enhydra.xmlc.verbose";
    private final String XMLC_PRINTACCESSMETHODS = "enhydra.xmlc.printAccessMethods";
    private final String XMLC_SAVE_OUTPUT = "enhydra.xmlc.saveOutputToFile";
    private final String XMLC_OUTPUT_FILENAME = "enhydra.xmlc.outputToFilename";
    private final String XMLC_MAP_PACKAGE_FROM = "enhydra.xmlc.map.package.fromFolder";
    private final String XMLC_MAP_PACKAGE_TO = "enhydra.xmlc.map.package.toPackage";
    private final String XMLC_MAP_PACKAGE_LENGTH = "enhydra.xmlc.map.package.length";
    private final String XMLC_MAP_SCOPE = "enhydra.xmlc.map.scope";
    private final String XMLC_MAP_IMAGES = "enhydra.xmlc.map.images";

    private final String CONF_MAKE = "enhydra.conf.make";
    private final String CONF_OVERWRITE = "enhydra.conf.overwrite";
    private final String CONF_OUTPUT    = "enhydra.conf.output";
    private final String CONF_TABLE_LENGTH  = "enhydra.conf.table.length";
    private final String CONF_TABLE_FIND    = "enhydra.conf.table.find";
    private final String CONF_TABLE_REPLACE = "enhydra.conf.table.replace";
    private final String CONF_STARTUP = "enhydra.conf.startup";
    private final String CONF_STARTUP_JAVA = "enhydra.conf.startup.java";
    private final String CONF_STARTUP_RUN = "enhydra.conf.startup.run";
    private final String CONF_STARTUP_CONF = "enhydra.conf.startup.conf";

    public final static int MAP_SCOPE_ALL      = 0;
    public final static int MAP_SCOPE_SELECTED = 1;
    public final static int MAP_SCOPE_NONE     = 2;


    abstract public OtterImageFileNode[] getAllImages();
    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public Object getNativeProject();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public String getName();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public boolean isCompileJavaAllowed();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public String getOutputDirectory();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public String getSourcePath();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public String[] getSourcePathArray();

    /**
     * Method declaration
     *
     *
     * @param pathAsArray
     *
     * @see
     */
    abstract public void setSourcePathArray(String[] pathAsArray);

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public String getClassPath();

    /**
     * Method declaration
     *
     *
     * @param property
     *
     * @return
     *
     * @see
     */
    abstract public String getProperty(String property);

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public OtterNodeFactory getNodeFactory();

    abstract public void configureRunClass();

    /**
     * Method declaration
     *
     *
     * @param property
     * @param value
     *
     * @see
     */
    abstract public void setProperty(String property, String value);
    abstract public void setProperty(String property, int value);

    public String getOptionFilename() {
      String filename = new String();
      File file = PropUtil.getFileRelativeToProject(
           this, OtterNode.OPTION_FILENAME);
      if ((file != null) && file.exists() && ( ! file.isDirectory())) {
         filename = file.getAbsolutePath();
      }
      return filename;
    }

    public void setOptionFilename(String filename) {
      PropUtil.putFileRelativeToProject(
         this, OtterNode.OPTION_FILENAME, filename);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isCompileJavaSelected() {
	boolean b = false;

	if (isCompileJavaAllowed()) {
	    b = readBooleanProperty(XMLC_COMPILE_JAVA, true);
	}

	return b;
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setCompileJavaSelected(boolean b) {
	writeBooleanProperty(XMLC_COMPILE_JAVA, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isDumpSelected() {
	return readBooleanProperty(XMLC_DUMP, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setDumpSelected(boolean b) {
	writeBooleanProperty(XMLC_DUMP, b);
    }

    public boolean isXMLCMake() {
	return readBooleanProperty(XMLC_MAKE, false);
    }

    public void setXMLCMake(boolean b) {
	writeBooleanProperty(XMLC_MAKE, b);
    }

    public boolean isMapImages() {
	return readBooleanProperty(XMLC_MAP_IMAGES, true);
    }

    public void setMapImages(boolean b) {
	writeBooleanProperty(XMLC_MAP_IMAGES, b);
    }


    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isParseInfoSelected() {
	return readBooleanProperty(XMLC_PARSEINFO, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setParseInfoSelected(boolean b) {
	writeBooleanProperty(XMLC_PARSEINFO, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isPrintAccessMethodsSelected() {
	return readBooleanProperty(XMLC_PRINTACCESSMETHODS, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setPrintAccessMethodsSelected(boolean b) {
	writeBooleanProperty(XMLC_PRINTACCESSMETHODS, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isPrintInfoSelected() {
	return readBooleanProperty(XMLC_PRINTINFO, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setPrintInfoSelected(boolean b) {
	writeBooleanProperty(XMLC_PRINTINFO, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isVerboseSelected() {
	return readBooleanProperty(XMLC_VERBOSE, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setVerboseSelected(boolean b) {
	writeBooleanProperty(XMLC_VERBOSE, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isKeepJavaSelected() {
	return readBooleanProperty(XMLC_KEEP_JAVA, true);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setKeepJavaSelected(boolean b) {
	writeBooleanProperty(XMLC_KEEP_JAVA, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isSaveOutputSelected() {
	return readBooleanProperty(XMLC_SAVE_OUTPUT, true);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setSaveOutputSelected(boolean b) {
	writeBooleanProperty(XMLC_SAVE_OUTPUT, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public int getMapScope() {
      String strScope;
      int intScope;
      strScope = getProperty(XMLC_MAP_SCOPE);
      intScope = PropUtil.stringToInt(strScope, MAP_SCOPE_ALL);
      return intScope;
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setMapScope(int i) {
	setProperty(XMLC_MAP_SCOPE, i);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getOutputFilename() {
	String out = null;

	out = getProperty(XMLC_OUTPUT_FILENAME);

	return out;
    }

    /**
     * Method declaration
     *
     *
     * @param n
     *
     * @see
     */
    public void setOutputFilename(String n) {
	setProperty(XMLC_OUTPUT_FILENAME, n);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String[][] getPackageMap() {
       String[][] map = PropUtil.getArrayProperty(this,
           XMLC_MAP_PACKAGE_LENGTH,
           XMLC_MAP_PACKAGE_FROM,
           XMLC_MAP_PACKAGE_TO);
       File from;
       for (int i = 0 ; i < map.length ; i++ ) {
         // make separators match current OS
         map[i][0] = map[i][0].replace('\\',File.separatorChar);
         map[i][0] = map[i][0].replace('/', File.separatorChar);
       }
       return map;
    }

    /**
     * Method declaration
     *
     *
     * @param map
     *
     * @see
     */
    public void putPackageMap(String[][] map) {
       PropUtil.putArrayProperty(this,
           XMLC_MAP_PACKAGE_LENGTH,
           XMLC_MAP_PACKAGE_FROM,
           XMLC_MAP_PACKAGE_TO,
           map);
    }

    public String[][] getDefaultConfTable() {
      return DEFAULT_CONF_TABLE;
    }

    public String[][] getConfTable() {
       String[][] confTable =
          PropUtil.getArrayProperty(this,
           CONF_TABLE_LENGTH,
           CONF_TABLE_FIND,
           CONF_TABLE_REPLACE);
      // replace relative paths in replaceWith column
      if (confTable.length == 0) {
        confTable = DEFAULT_CONF_TABLE;
      }
      for (int i = 0 ; i < confTable.length ; i++ ) {
       confTable[i][1] =
          PropUtil.compressPathRelativeToProject(this, confTable[i][1]);
      }
      return confTable;
    }

    /**
     * Method declaration
     *
     *
     * @param map
     *
     * @see
     */
    public void putConfTable(String[][] confTable) {
       // replace relative paths in replaceWith column
       for (int i = 0 ; i < confTable.length ; i++ ) {
        confTable[i][1] =
           PropUtil.compressPathRelativeToProject(this, confTable[i][1]);
       }
       PropUtil.putArrayProperty(this,
           CONF_TABLE_LENGTH,
           CONF_TABLE_FIND,
           CONF_TABLE_REPLACE,
           confTable);
    }


    /**
     * Method declaration
     *
     *
     * @param property
     * @param def
     *
     * @return
     *
     * @see
     */
    private boolean readBooleanProperty(String property, boolean def) {
	boolean flag = def;
	String  in = getProperty(property);

	return PropUtil.stringToBoolean(in, def);
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param b
     *
     * @see
     */
    private void writeBooleanProperty(String property, boolean b) {
	String flag = b ? "true" : "false";

	setProperty(property, flag);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getGenerateToDirectory() {
	String genToDir = null;
        if (readBooleanProperty("GenerateSourceToOutDir", true)) {
   	   genToDir = getOutputDirectory()
                      + File.separator
	              + "Generated Source";
	}
	return genToDir;
    }


  public boolean isConfOverwriteSelected() {
    String in = getProperty(CONF_OVERWRITE);
    return PropUtil.stringToBoolean(in, false);
  }

  public void setConfOverwriteSelected(boolean b) {
   setProperty(CONF_OVERWRITE, PropUtil.booleanToString(b));
  }

  public String getConfOutputDirectory() {
    File file = new File(getFilename());
    String out = file.getParent() + File.separator + "output";

    file = PropUtil.getFileRelativeToProject(
       this, CONF_OUTPUT);
    if ((file != null) &&
        (! file.isFile()) &&
        file.exists()) {
      out = file.getAbsolutePath();
    }
    return out;
  }

  public void setConfOutputDirectory(String directory) {
    PropUtil.putFileRelativeToProject(
       this, CONF_OUTPUT, directory);
  }

  public boolean isConfStartupSelected() {
    String in = getProperty(CONF_STARTUP);
    return PropUtil.stringToBoolean(in, true);
  }

  public void setConfStartupSelected(boolean b) {
   setProperty(CONF_STARTUP, PropUtil.booleanToString(b));
  }

  public boolean isConfStartupRun() {
    String in = getProperty(CONF_STARTUP_RUN);
    return PropUtil.stringToBoolean(in, ! isCompileJavaAllowed());
  }

  public void setConfStartupRun(boolean b) {
   setProperty(CONF_STARTUP_RUN, PropUtil.booleanToString(b));
  }

  public boolean isConfStartupJava() {
    String in = getProperty(CONF_STARTUP_JAVA);
    return PropUtil.stringToBoolean(in, isCompileJavaAllowed());
  }

  public void setConfStartupJava(boolean b) {
   setProperty(CONF_STARTUP_JAVA, PropUtil.booleanToString(b));
  }

  public String getConfStartupConf() {
    String conf = getProperty(CONF_STARTUP_CONF);
    if (conf == null || conf.trim().length() == 0) {
      // set default;
      conf = "./output/multiserver.conf";
      setProperty(CONF_STARTUP_CONF, conf);
    }
    File file = PropUtil.getFileRelativeToProject(
       this, CONF_STARTUP_CONF);
    if ((file != null) && (! file.isDirectory())) {
      conf = file.getAbsolutePath();
    }
    return conf;

  }

  public void setConfStartupConf(String conf) {
    PropUtil.putFileRelativeToProject(
       this, CONF_STARTUP_CONF, conf);
  }

    public boolean isConfMake() {
	return readBooleanProperty(CONF_MAKE, false);
    }

    public void setConfMake(boolean b) {
	writeBooleanProperty(CONF_MAKE, b);
    }

}



