/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: OtterXMLCNode.java,v 1.17.2.5 2000/03/22 09:30:20 pmahar Exp $
 */

package org.enhydra.ide.jbuilder.common.node;

import org.enhydra.ide.jbuilder.common.PropUtil;
import org.enhydra.ide.jbuilder.common.node.OtterProject;
import org.enhydra.xml.xmlc.compiler.Parse;
import org.enhydra.xml.xmlc.XMLCException;
import org.enhydra.xml.xmlc.commands.xmlc.XMLCOptions;
import org.enhydra.xml.xmlc.commands.options.OptionsParser;
import org.enhydra.xml.xmlc.commands.options.OptionSet;

import java.io.File;
import java.util.Vector;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

public class OtterXMLCNode implements OtterHTMLFileNode {
    public final static int CLASS_NAME_DEFAULT = 1;
    public final static int CLASS_NAME_CUSTOM = 2;
    public final static int CLASS_NAME_MAPPED = 3;
    private final String CUSTOM_CLASS_NAME = "enhydra.xmlc.customClassName";
    private final String CLASS_NAME_TYPE   = "enhydra.xmlc.classNameType";

    private boolean parseFailure = false;
    private Options      options;
    private OtterHTMLFileNode fileNode;

    /**
     * Create a node to use when calling XMLC.
     *
     */
  public OtterXMLCNode(OtterHTMLFileNode f) throws
     XMLCException {
    fileNode = f;
    initOptions();
  }

   public String getOptionFilename() {
      return fileNode.getOptionFilename();
    }

   public void setOptionFilename(String n) {
      fileNode.setOptionFilename(n);
    }


  public boolean isSelected() {
    return fileNode.isSelected();
  }

  public void setSelected(boolean b) {
    fileNode.setSelected(b);
  }

  public char getFileSeparatorChar() {
    return fileNode.getFileSeparatorChar();
  }

  public String getFilename() {
    return fileNode.getFilename();
  }

  public String getJavacDirectory() {
    return fileNode.getJavacDirectory();
  }

  public void setJavacDirectory(String s) {
      fileNode.setJavacDirectory(s);
   }

  public String getCustomClassName() {
    String custom = getProperty(CUSTOM_CLASS_NAME);
    if (custom == null) {
      custom = new String();
    }
    return custom;
  }

  public void setCustomClassName(String n) {
    setProperty(CUSTOM_CLASS_NAME, n);
  }


  public Object getNativeNode() {
    return fileNode.getNativeNode();
  }

  public void setNativeNode(Object o) {
    fileNode.setNativeNode(o);
  }

  public OtterProject getProject() {
    return fileNode.getProject();
  }


  public String getProperty(String n) {
    return fileNode.getProperty(n);
  }

  public void setProperty(String n, String v) {
    fileNode.setProperty(n, v);
  }

  public void setProperty(String n, int v) {
    fileNode.setProperty(n, v);
  }


  public int getClassNameType() {
    // When getting the project scope overrides
    // local type.
    int type;
    int projectScope;
    String stringType = getProperty(CLASS_NAME_TYPE);
    projectScope = getProject().getMapScope();
    type         = PropUtil.stringToInt(stringType, CLASS_NAME_MAPPED);
    switch (type) {
      case CLASS_NAME_DEFAULT:
      case CLASS_NAME_CUSTOM:
           if (projectScope == OtterProject.MAP_SCOPE_ALL) {
              type = CLASS_NAME_MAPPED;
              setProperty(CLASS_NAME_TYPE, type);
           }
           break;
      case CLASS_NAME_MAPPED:
           if (projectScope == OtterProject.MAP_SCOPE_NONE) {
              type = CLASS_NAME_DEFAULT;
              setProperty(CLASS_NAME_TYPE, type);
           }
           break;
    }
    return type;
  }


  public void setClassNameType(int newType) {
    // If conflict exists with project scope,
    // change project scope.
    int oldType       = getClassNameType();
    int projectScope = getProject().getMapScope();
    setProperty(CLASS_NAME_TYPE, newType);
    syncProjectToType(newType);
    if ((newType != oldType) && (newType != CLASS_NAME_CUSTOM)) {
      initClassName();
    }
  }

  private void syncProjectToType(int type) {
    int projectScope = getProject().getMapScope();
    switch (type){
      case CLASS_NAME_CUSTOM:
      case CLASS_NAME_DEFAULT:
           if (projectScope == OtterProject.MAP_SCOPE_ALL) {
               getProject().setMapScope(OtterProject.MAP_SCOPE_SELECTED);
           }
           break;
      case CLASS_NAME_MAPPED:
           if (projectScope == OtterProject.MAP_SCOPE_NONE) {
               getProject().setMapScope(OtterProject.MAP_SCOPE_SELECTED);
           }
          break;
    }
  }


    /**
     * Save Exception information for reporting. This lets exceptions from
     * multiple nodes to be reported in a batch.
     *
     * @param e
     * An exception that occured while invoking XMLC.
     */
    public void setException(Exception e) {
	fileNode.setException(e);
    }

    /**
     * Get an exception that occured when invoking XMLC. Null if the node
     * has not been compiled or was compiled without error.
     */
    public Exception getException() {
	return fileNode.getException();
    }

    /**
     * Get the XMLC options selected this HTML file.
     */
    public Options getOptions() {
	return options;
    }

    /**
     * Add the generated Java source file to the IDE project. Do not call
     * this when using Win32 JBuilder as it can't handle the width of source
     * lines generated by XMLC.
     */
    public void replaceGeneratedSource() {
	if ((options.getJavaClassSource().exists())
		&& (! getProject().isCompileJavaAllowed())) {
           String javaSource = options.getJavaClassSource().getAbsolutePath();
           OtterNodeFactory factory = getProject().getNodeFactory();
           factory.replaceGeneratedSource(getProject(), fileNode, javaSource);
	}
    }

   public void initProjectOptions() {
	options.setKeep(getProject().isKeepJavaSelected());
	options.setNoCompile(getProject().isCompileJavaAllowed() &&
                             getProject().isCompileJavaSelected());

	// loging options
//	options.setParser(Parse.HTML_SWING_PARSER);
        options.setParseInfo(getProject().isParseInfoSelected());
        options.setDump(getProject().isDumpSelected());
	options.setVerbose(getProject().isVerboseSelected());
	options.setPrintAccessMethods(getProject().isPrintAccessMethodsSelected());

	// Standard JAVAC options
	// compilition options
        if (fileNode.getProject().isCompileJavaAllowed()) {
           options.setJavac("bcjhide.exe");  // Win32 JBuilder and JDeveloper
	}
   }


    /**
     * Initialize the XMLC options.
     *
     * @throws XMLCException
     * Thrown when options are invalid.
     *
     * TODO: Mark is changing stuff that might change this method.
     * TODO: Add support of xmlc file to do advanced options (-implements,
     * -urlmapping, -urlsetting, javacflag, -javacopt, and -docout).
     */
    private void initOptions() throws XMLCException {
        Vector opFiles = new Vector();
	options = new Options();
	initClassName();
        initProjectOptions();
        File sourceDoc = null;
        sourceDoc = new File(getFilename());
        options.setSourceDocument(sourceDoc, true);
        initJavaSourceFile();
        options.complete();
        if (fileNode.getProject().getOptionFilename() != null) {
          // must insert project first so it can be overriden
          File f = new File(fileNode.getProject().getOptionFilename());
          if (f.exists() && f.isFile() && f.getName().toLowerCase().endsWith(".xmlc")) {
            opFiles.addElement(f.getAbsolutePath());
          }
        }
        if (getOptionFilename() != null) {
          File f = new File(getOptionFilename());
          if (f.exists() && f.isFile() && f.getName().toLowerCase().endsWith(".xmlc")) {
            opFiles.addElement(f.getAbsolutePath());
          }
        }
        if (opFiles.size() > 0 && (! parseFailure)) {
          OptionsParser op = getNewOptionsParser(options);
          String args[] = new String[opFiles.size()];
          for (int i = 0 ; i < args.length ; i++ ) {
            args[i] = (String) opFiles.elementAt(i);
          }
          try {
            callParse(op, options, args);
          } catch (java.io.IOException e) {
            e.printStackTrace();
            parseFailure = true;
            initOptions();
          }
        }
      }

   // This method allows us to create a new instance using the
   // 2.3 or 3.0 XMLC API.
   private OptionsParser getNewOptionsParser(Options options) {
      OptionsParser parser = null;
      try {
        Class opClass = Class.forName("org.enhydra.xml.xmlc.commands.options.OptionsParser");
        Constructor[] opCon = opClass.getConstructors();
        Class[] params = opCon[0].getParameterTypes();
        Object[] values = new Object[0];
        if (params.length == 1) {
          values = new Object[1];
          values[0] = options;
        }
        parser = (OptionsParser) opCon[0].newInstance(values);
      } catch (Exception e) {
        parser = null;
        e.printStackTrace();
      }
      return parser;
   }

   private void callParse(OptionsParser parser,
              Options options,
              String[] args) throws java.io.IOException {
     Class    opClass = parser.getClass();
     Class[]  params  = new Class[1];
     Object[] values  = null;
     Method   method  = null;
     params[0] = args.getClass();
     try {
       try {
         method = opClass.getMethod("parse",params);
         values = new Object[1];
         values[0] = args;
      } catch (NoSuchMethodException e) {
         params = new Class[2];
         params[0] = Class.forName("org.enhydra.xml.xmlc.commands.options.OptionSet");
         params[1] = args.getClass();
         method = opClass.getMethod("parse",params);
         values = new Object[2];
         values[0] = options;
         values[1] = args;
      }
      method.invoke(parser, values);
    } catch (Exception e) {
      e.printStackTrace();
    }

    // op.parse(args); // 3.0
    // op.parse(options, args); //2.3

   }


     private void initJavaSourceFile() {
	StringBuffer filename = new StringBuffer();
        int index = -1;
        filename.append(getGenToDirectory().toString());
        filename.append("" + getProject().getFileSeparatorChar());
        String className = options.getClassName();
        if (options.getPackageName() != null) {
          index = className.indexOf(options.getPackageName());
        }
        if (index > -1) {
          className = className.substring(options.getPackageName().length()+1);
        }
        index = className.lastIndexOf(".");
        if (index > -1) {
          className = className.substring(index + 1);
        }
        filename.append(className);
	filename.append(".java");
	File f = new File(filename.toString());
	options.setJavaClassSource(f);
    }

    /**
     * Get the generate to directory and create the directory if needed.
     * This is where the Java source files are created.
     */
    private File getGenToDirectory() {
	String genTo = getGenTo();
	File   genDir = new File(genTo);

	if (genDir.exists()) {
	    if (!genDir.isDirectory()) {
		System.err.println("Unable to write to directory: " + genTo);
	    }
	} else {
	    genDir.mkdirs();
	}

	return genDir;
    }

    /**
     * Get the generate to directory as a string. This is where XMLC will create
     * Java source files.
     */
    private String getGenTo() {
	String genTo      = getProject().getGenerateToDirectory();
	File   sourceFile = null;
	if (genTo == null) {
	    sourceFile = new File(getFilename());
	    genTo      = sourceFile.getParent();
	} else {
            genTo = genTo + File.separator + getPackagePath();
        }
	return genTo;
    }

    private String getPackagePath() {
      String packPath = new String();
      if (options.getPackageName() != null) {
        packPath = options.getPackageName();
      } else if (options.getClassName() != null) {
        int index = options.getClassName().lastIndexOf('.');
        if (index > -1) {
           packPath = options.getClassName().substring(0,index);
        }
      }
      packPath = packPath.replace('.',File.separatorChar);
      return packPath;
    }

    /**
     * Create a class name for the given HTML file and map the
     * package name as needed. Mapping allows users to  specify
     * a package name to use for all the HTML files in a directory.
     */
    private void initClassName() {
        Mapper mapper = new Mapper(getProject());
        String className = mapper.getClassName(this);
        options.setClassName(className);
    }

    /**
     * Get the class file that we expect after XMLC has generated
     * a java file and called javac to compile it.
     */
    public File getClassFile() {
	String cfName = getProject().getOutputDirectory() +
                      "" + getProject().getFileSeparatorChar()
			+ options.getClassName();

	cfName = cfName.replace('.', getProject().getFileSeparatorChar());
	cfName = cfName + ".class";

	File cf = new File(cfName);

	return cf;
    }

    /**
     * True if we can find the class file that we expect XMLC to create.
     * This is used to in reporting compilation results.
     */
    public boolean isNewClassFound() {
	boolean newFound = false;

	if (!options.getNoCompile()) {
	    newFound = getClassFile().exists();
	}

	return newFound;
    }

    /**
     * True if we can find the Java source file that we expect XMLC to create.
     * This is used to in reporting compilation results.
     */
    public boolean isNewJavaFound() {
	boolean newFound = false;

	if (options.getKeep()) {
	    newFound = options.getJavaClassSource().exists();
	}

	return newFound;
    }

    /**
     * Delete the class and java files that we expect to get recreated by XMLC.
     */
    private void clearFiles() {
	if (options.getKeep() && options.getJavaClassSource().exists()) {
	    options.getJavaClassSource().delete();
	}

	File cf = getClassFile();

	if ((!options.getNoCompile()) && (cf.exists())) {
	    cf.delete();
	}
    }

    public void preCompile() {
      fileNode.preCompile();
    }


    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    public class Options extends XMLCOptions implements OptionSet {
      private File javaClassSource = null;
      private File sourceDoc = null;

      public void setJavaClassSource(File f) {
        javaClassSource = f;
      }

      public File getJavaClassSource() {
        return javaClassSource;
      }

      public void setSourceDocument(File f, boolean real) {
        if (real) {
          sourceDoc = f;
          setSourceDocument(sourceDoc);
        }
      }

      public void setSourceDocument(File f) {
        super.setSourceDocument(sourceDoc);
      }

      public File getSourceDocument() {
        return sourceDoc;
      }
  }

}


