/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: OutputPanel.java,v 1.1.2.1 2000/03/20 08:20:53 pmahar Exp $
 */
package org.enhydra.ide.jbuilder.common.wizard;

import org.enhydra.ide.jbuilder.common.node.OtterProject;
import org.enhydra.ide.jbuilder.common.capture.WriteEvent;
import org.enhydra.ide.jbuilder.common.capture.WriteListener;
import org.enhydra.devsys.swing.ExtensionFilter;
import org.enhydra.devsys.swing.GridBagConstraintsX;
import javax.swing.*;
import javax.swing.text.BadLocationException;
import java.awt.*;
import java.awt.event.*;
import java.beans.Beans;
import java.io.File;
import java.io.FileWriter;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class OutputPanel extends JPanel implements WriteListener {
    private LocalButtonListener buttonListener = null;
    private OtterProject	project = null;
    private JPanel		panelFile;
    private JScrollPane		scrollOutput;
    private JCheckBox		checkSave;
    private JTextArea		textAreaOutput;
    private JTextField		textFile;
    private JButton		buttonBrowse;
    private GridBagLayout       layoutFile;
    private GridBagLayout       layoutMain;

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public OutputPanel() {
	try {
	    jbInit();
	    pmInit();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Method declaration
     *
     *
     * @param event
     *
     * @see
     */
    public void onWrite(WriteEvent event) {
	addOutput(event.getString());
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    protected OtterProject getProject() {
	return project;
    }

    /**
     * Method declaration
     *
     *
     * @param p
     *
     * @see
     */
    protected void setProject(OtterProject p) {
	project = p;

	String outFile = project.getOutputFilename();

	if (outFile == null || outFile.trim().length() < 1) {
	    checkSave.setSelected(false);
	    textFile.setText(new String());
	} else {
	    textFile.setText(project.getOutputFilename().trim());
	    refreshCheckSave();
	}
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void refreshCheckSave() {
	if (project.getOutputFilename().trim().length() > 0) {
	    checkSave.setSelected(project.isSaveOutputSelected());
	    checkSave.setEnabled(true);
	} else {
	    checkSave.setSelected(false);
	    checkSave.setEnabled(false);
	}
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void pmInit() {
	buttonListener = new LocalButtonListener();

	buttonBrowse.addActionListener(buttonListener);
    }

    /**
     * Method declaration
     *
     *
     * @throws Exception
     *
     * @see
     */
    private void jbInit() throws Exception {
	panelFile = (JPanel) Beans.instantiate(getClass().getClassLoader(),
					       JPanel.class.getName());
	textAreaOutput =
	    (JTextArea) Beans.instantiate(getClass().getClassLoader(),
					  JTextArea.class.getName());

	textAreaOutput.setEditable(false);
	textAreaOutput.setRows(15);

	scrollOutput = new JScrollPane(textAreaOutput);
	checkSave = (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
						  JCheckBox.class.getName());
	textFile = (JTextField) Beans.instantiate(getClass().getClassLoader(),
						  JTextField.class.getName());
	buttonBrowse =
	    (JButton) Beans.instantiate(getClass().getClassLoader(),
					JButton.class.getName());
	layoutFile =
	    (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
					      GridBagLayout.class.getName());
	layoutMain =
	    (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
					      GridBagLayout.class.getName());

	buttonBrowse.setText("Browse...");
	panelFile.setLayout(layoutFile);
	checkSave.setText("Output to log file");
	textFile.setBackground(SystemColor.controlLtHighlight);
	textFile.setEnabled(false);
	textFile.setPreferredSize(new Dimension(150, 21));
	panelFile.add(checkSave,
		      new GridBagConstraintsX(0, 0, 1, 1, 0.0, 0.0,
					     GridBagConstraints.WEST,
					     GridBagConstraints.BOTH,
					     new Insets(5, 5, 5, 5), 14, 0));
	panelFile.add(textFile,
		      new GridBagConstraintsX(1, 0, 1, 1, 0.5, 0.5,
					     GridBagConstraints.CENTER,
					     GridBagConstraints.HORIZONTAL,
					     new Insets(5, 5, 5, 5), 14, 0));
	panelFile.add(buttonBrowse,
		      new GridBagConstraintsX(2, 0, 1, 1, 0.5, 0.5,
					     GridBagConstraints.WEST,
					     GridBagConstraints.NONE,
					     new Insets(5, 5, 5, 5), 6, 0));
	this.setLayout(layoutMain);
	this.add(scrollOutput,
		 new GridBagConstraintsX(0, 0, 1, 1, 0.75, 0.75,
					GridBagConstraints.WEST,
					GridBagConstraints.BOTH,
					new Insets(8, 12, 0, 0), 0, 0));
	this.add(panelFile,
		 new GridBagConstraintsX(0, 1, 1, 1, 0.25, 0.25,
					GridBagConstraints.CENTER,
					GridBagConstraints.BOTH,
					new Insets(0, 12, 0, 0), 0, 0));
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    protected void scrollToBottom() {
	Runnable runSwing = new Runnable() {

	    /**
	     * Method declaration
	     *
	     *
	     * @see
	     */
	    public void run() {
		scrollOutput.getVerticalScrollBar().setValue(scrollOutput.getVerticalScrollBar().getMaximum());
	    }

	};

	try {
	    if (SwingUtilities.isEventDispatchThread()) {
		runSwing.run();
	    } else {
		SwingUtilities.invokeAndWait(runSwing);
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}

	try {
	    Thread.sleep(500);
	} catch (InterruptedException e) {

	    //
	}
    }

    /**
     * Method declaration
     *
     *
     * @param output
     *
     * @see
     */
    protected void addOutput(final String output) {
	Runnable runAdd = new Runnable() {

	    /**
	     * Method declaration
	     *
	     *
	     * @see
	     */
	    public void run() {
		int start = 0;
		int end = 0;

		if (textAreaOutput.getLineCount() > 2000) {
		    try {
			start = textAreaOutput.getLineStartOffset(0);
			end = textAreaOutput.getLineEndOffset(0);
		    } catch (BadLocationException e) {
			start = 0;
			end = 0;
		    }

		    textAreaOutput.replaceRange(null, start, end);
		}

		textAreaOutput.append(output);
		scrollOutput.getVerticalScrollBar().setValue(scrollOutput.getVerticalScrollBar().getMaximum());
		textAreaOutput.repaint();
		repaint();
	    }

	};

	try {
	    if (SwingUtilities.isEventDispatchThread()) {
		runAdd.run();
	    } else {
		SwingUtilities.invokeAndWait(runAdd);
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}

	if ((project.isSaveOutputSelected())
		&& (project.getOutputFilename() != null)
		&& (project.getOutputFilename().trim().length() > 0)) {
	    try {
		FileWriter fw = new FileWriter(project.getOutputFilename(),
					       true);

		fw.write(output);    // line + new Character((char)13) + new Character((char)10));
		fw.close();
	    } catch (java.io.IOException e) {
		e.printStackTrace();
	    }
	}
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void buttonFileAction() {
	JFileChooser    chooser;
	ExtensionFilter filter;
	File		file;
	String		fileDir = new String();
	String		fileName = new String();

	filter = new ExtensionFilter();

	filter.addExtension("txt");
	filter.setDescriptionTitle("Enhydra log file");

	chooser = new JFileChooser();

	if (project.getOutputFilename() != null) {
	    file = new File(project.getOutputFilename());

	    chooser.setCurrentDirectory(file);
	}

	chooser.setFileFilter(filter);
	chooser.setDialogTitle("Select a log file");
	chooser.setApproveButtonText("OK");

	int v = chooser.showOpenDialog(this);

	this.requestFocus();
	buttonBrowse.requestFocus();

	if (v == JFileChooser.APPROVE_OPTION
		&& chooser.getSelectedFile() != null) {
	    fileDir = chooser.getCurrentDirectory().toString();
	    fileDir = stripSlash(fileDir);
	    fileName = chooser.getSelectedFile().getName();

	    textFile.setText(fileDir + File.separator + fileName);
	    project.setOutputFilename(textFile.getText().trim());
	    refreshCheckSave();
	}

	chooser.removeAll();

	chooser = null;
    }

    /**
     * Method declaration
     *
     *
     * @param dir
     *
     * @return
     *
     * @see
     */
    private String stripSlash(String dir) {
	String stripped = new String(dir);

	if (dir != null) {
	    if (dir.length() > 0) {
		if (dir.endsWith(File.separator)) {
		    stripped = dir.substring(0, dir.length() - 1);
		}
	    }
	}

	return stripped;
    }

    /**
     * Method declaration
     *
     *
     * @param event
     *
     * @see
     */
    private void checkSaveStateChanged(ItemEvent event) {
	boolean selected = (event.getStateChange() == ItemEvent.SELECTED);

	project.setSaveOutputSelected(selected);
    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class LocalItemListener implements ItemListener {

	/**
	 * Method declaration
	 *
	 *
	 * @param event
	 *
	 * @see
	 */
	public void itemStateChanged(ItemEvent event) {
	    Object source = event.getSource();

	    if (source == checkSave) {
		checkSaveStateChanged(event);
	    }
	}

    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class LocalButtonListener implements ActionListener {

	/**
	 * Method declaration
	 *
	 *
	 * @param event
	 *
	 * @see
	 */
	public void actionPerformed(ActionEvent event) {
	    Object source = event.getSource();

	    if (source == buttonBrowse) {
		buttonFileAction();
	    }
	}

    }

}



/*--- formatting done in "Sun Java Convention" style on 01-16-2000 ---*/

