/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *re
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: ConfNodePropertyPanel.java,v 1.1.2.4 2000/03/26 21:33:39 pmahar Exp $
 */
package org.enhydra.ide.jbuilder.common.wizard.conf;

import org.enhydra.ide.jbuilder.common.wizard.AbstractPropertyPanel;
import org.enhydra.ide.jbuilder.common.node.OtterConfFileNode;
import org.enhydra.ide.jbuilder.common.node.OtterNode;
import org.enhydra.ide.jbuilder.common.node.OtterProject;
import org.enhydra.devsys.swing.SwingUtil;
import org.enhydra.devsys.swing.SwingTableSelectionEvent;
import org.enhydra.devsys.swing.SwingTableSelectionListener;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.File;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.table.*;

public class ConfNodePropertyPanel extends JPanel
     implements ListSelectionListener {


  private OtterNode node = null;
  private GridBagLayout layoutMain;
  private JCheckBox checkConfigure;
  private JCheckBox checkOverwrite;
  private JPanel panelClassName;
  private GridBagLayout layoutClassName;
  private JPanel panelDirectory;
  private TitledBorder borderClassName;
  private LocalButtonListener buttonListener;
  private JTable	       tableConf;
  private JScrollPane	       scrollTable;
  private LocalTableModel      tableModel = new LocalTableModel();
  private JTextField textDirectory;
  private TitledBorder borderDirectory;
  private GridBagLayout layoutDirectory;
  private JButton buttonSet;
  private JButton buttonAdd;
  private JButton buttonEdit;
  private JButton buttonRemove;
  private JButton buttonDefault;
  private int currentSelectionIndex = -1;
  private Vector swingTableSelectionListeners = new Vector();

  public ConfNodePropertyPanel() {
    try {
      jbInit();
      pmInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

    /**
     * ListSelectionListener event
     */
    public void valueChanged(ListSelectionEvent e) {
	ListSelectionModel lsm = (ListSelectionModel) e.getSource();

	if (lsm.isSelectionEmpty()) {
	    currentSelectionIndex = -1;
	} else {
	    currentSelectionIndex = lsm.getMinSelectionIndex();
	}
        SwingTableSelectionListener listener = null;
        for (int i = 0 ; i < swingTableSelectionListeners.size() ; i++ ) {
          listener = (SwingTableSelectionListener) swingTableSelectionListeners.elementAt(i);
          listener.onSwingTableSelection(
             new SwingTableSelectionEvent(this, currentSelectionIndex));
        }
    }

  public OtterNode getNode() {
    return node;
  }

  public void setNode(OtterNode n) {
    node = n;
  }


  private void pmInit() throws Exception {
    buttonListener = new LocalButtonListener();
    buttonSet.addActionListener(buttonListener);
    buttonDefault.addActionListener(buttonListener);
    buttonRemove.addActionListener(buttonListener);

    tableModel = new LocalTableModel();
    tableConf.setModel(tableModel);
//	tableConf.setDefaultRenderer(String.class, new CellRendererWithToolTip());
	tableModel.addTableModelListener(tableConf);
	tableConf.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	tableConf.getTableHeader().setUpdateTableInRealTime(false);
	tableConf.getTableHeader().setReorderingAllowed(false);
	ListSelectionModel rowSM = tableConf.getSelectionModel();

	rowSM.addListSelectionListener(this);

  }

  private void jbInit() throws Exception {
    checkConfigure = (JCheckBox) Beans.instantiate(getClass().getClassLoader(), JCheckBox.class.getName());
    checkOverwrite    = (JCheckBox) Beans.instantiate(getClass().getClassLoader(), JCheckBox.class.getName());

    layoutMain = (GridBagLayout) Beans.instantiate(getClass().getClassLoader(), GridBagLayout.class.getName());
    layoutDirectory = (GridBagLayout) Beans.instantiate(getClass().getClassLoader(), GridBagLayout.class.getName());

    panelClassName = (JPanel) Beans.instantiate(getClass().getClassLoader(), JPanel.class.getName());
    layoutClassName = (GridBagLayout) Beans.instantiate(getClass().getClassLoader(), GridBagLayout.class.getName());
    panelDirectory = (JPanel) Beans.instantiate(getClass().getClassLoader(), JPanel.class.getName());
      buttonSet = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
      textDirectory = (JTextField) Beans.instantiate(getClass().getClassLoader(), JTextField.class.getName());


    tableConf = (JTable) Beans.instantiate(getClass().getClassLoader(), JTable.class.getName());
    buttonAdd = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    buttonEdit = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    buttonRemove = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    buttonDefault = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    checkConfigure.setText("Generate .conf");
    checkConfigure.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        enableUI(checkConfigure.isSelected());
      }
    });

    checkOverwrite.setToolTipText("");
    checkOverwrite.setText("Overwrite if .conf already exists in output");
    panelClassName.setLayout(layoutClassName);
    borderClassName = BorderFactory.createTitledBorder("Replace text");
    panelClassName.setBorder(borderClassName);

    scrollTable = new JScrollPane(tableConf);
    tableConf.setToolTipText(new String());
    tableConf.sizeColumnsToFit(JTable.AUTO_RESIZE_ALL_COLUMNS);
    tableConf.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
    tableConf.setColumnSelectionAllowed(false);
    scrollTable.setMinimumSize(new Dimension(100, 100));
    scrollTable.setPreferredSize(new Dimension(100, 100));
    buttonAdd.setText("Add..");
    buttonEdit.setText("Edit...");
    buttonRemove.setText("Remove...");
    buttonDefault.setText("Default...");
    scrollTable.getViewport().add(tableConf, BorderLayout.CENTER);


    panelClassName.add(scrollTable, new GridBagConstraints(3, 0, 4, 1, 0.8, 0.8
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
    panelClassName.add(buttonAdd, new GridBagConstraints(3, 1, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    panelClassName.add(buttonEdit, new GridBagConstraints(4, 1, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    panelClassName.add(buttonRemove, new GridBagConstraints(5, 1, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
    panelClassName.add(buttonDefault, new GridBagConstraints(6, 1, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 5, 2, 5), 0, 0));


      borderDirectory = BorderFactory.createTitledBorder("Output directory");
      textDirectory.setMinimumSize(new Dimension(100, 21));
    textDirectory.setPreferredSize(new Dimension(100, 21));
    textDirectory.setEditable(false);
      buttonSet.setText("Set...");

      panelDirectory.setLayout(layoutDirectory);
      panelDirectory.setBorder(borderDirectory);
      panelDirectory.add(textDirectory, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.1
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 100, 0));
      panelDirectory.add(buttonSet, new GridBagConstraints(1, 0, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));


    this.setLayout(layoutMain);
    this.add(checkConfigure, new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
    this.add(checkOverwrite, new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 20, 5, 5), 0, 0));
    this.add(panelClassName, new GridBagConstraints(0, 2, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 15, 3, 5), 0, 0));
    this.add(panelDirectory, new GridBagConstraints(0, 3, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 15, 5, 5), 0, 0));
  }

  public void readProperties() {
    String stringIn = null;
    File   fileIn   = null;
    OtterProject project = node.getProject();
    if (node instanceof OtterConfFileNode) {
      OtterConfFileNode confNode = (OtterConfFileNode) node;
      checkConfigure.setSelected(confNode.isSelected());
      enableUI(checkConfigure.isSelected());
    } else {
      checkConfigure.setVisible(false);
      enableUI(true);
    }
    checkOverwrite.setSelected(project.isConfOverwriteSelected());
    putConfTable(project.getConfTable());
    textDirectory.setText(project.getConfOutputDirectory());
  }

  public void writeProperties() {
    OtterProject project = node.getProject();
    if (node instanceof OtterConfFileNode) {
      OtterConfFileNode confNode = (OtterConfFileNode) node;
      confNode.setSelected(checkConfigure.isSelected());
    }
    project.setConfOverwriteSelected(checkOverwrite.isSelected());
    project.putConfTable(getConfTable());
    project.setConfOutputDirectory(textDirectory.getText());
  }

  private String[][] getConfTable() {
    String[][] confTable = null;
    LocalRow   row       = null;
    int length = tableModel.getRowCount();
    confTable = new String[length][2];
    for (int i = 0 ; i < length ; i++ ) {
      row = tableModel.getRow(i);
      confTable[i][0] = row.getFind();
      confTable[i][1] = row.getReplaceWith();
    }
    return confTable;
  }

  private void putConfTable(String[][] confTable) {
    int length = tableModel.getRowCount();
    tableModel.removeAllRows();
    for (int i = 0 ; i < confTable.length ; i++) {
      tableModel.addRow(confTable[i][0], confTable[i][1]);
    }
    tableConf.updateUI();
  }

  private void enableUI(boolean enable) {
    checkOverwrite.setEnabled(enable);
    tableConf.setEnabled(enable);
    textDirectory.setEnabled(enable);
    buttonAdd.setEnabled(enable);
    buttonEdit.setEnabled(enable);
    buttonRemove.setEnabled(enable);
    buttonDefault.setEnabled(enable);
    buttonSet.setEnabled(enable);
  }

    private void browseForDirectory() {
       File d = new File(textDirectory.getText());
       d = SwingUtil.getDirectoryChoice(this, d,
  	  "Select .conf output directory");
       if (d != null && d.exists() && d.isDirectory()) {
         textDirectory.setText(d.getAbsolutePath());
       }
    }

    private void defaultTable() {
      int result = JOptionPane.YES_OPTION;
      if (tableModel.getRowCount() > 0) {
        result = JOptionPane.showConfirmDialog(this,
          "Replace current set with default?",
          "Switch to Default", JOptionPane.YES_NO_OPTION);
      }
      if (result == JOptionPane.YES_OPTION) {
         OtterProject project = node.getProject();
         putConfTable(project.getDefaultConfTable());
//        putConfTable(project.
//..        OtterProject.DEFAULT_CONF_TABLE);
      }
    }

    private void removeCurrent() {
      int result = 0;
      if (currentSelectionIndex > -1 &&
          currentSelectionIndex < tableModel.getRowCount()) {
        result = JOptionPane.showConfirmDialog(this,
          "Remove current replacement?",
          "Confirm Remove", JOptionPane.YES_NO_OPTION);
        if (result == JOptionPane.YES_OPTION) {
          tableModel.removeRow(currentSelectionIndex);
          tableConf.updateUI();
        }
      }
    }

    private class LocalButtonListener implements ActionListener {
      public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source == buttonSet) {
          browseForDirectory();
        } else if (source == buttonDefault) {
          defaultTable();
        } else if (source == buttonRemove) {
          removeCurrent();
        }
      }
    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class LocalTableModel extends AbstractTableModel {
	private Vector rowVector = new Vector();

	/**
	 * Constructor declaration
	 *
	 *
	 * @see
	 */
	public LocalTableModel() {}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public int getRowCount() {
	    int count = 0;

	    if (rowVector != null) {
		count = rowVector.size();
	    }

	    return count;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public int getColumnCount() {
	    return 2;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param columnIndex
	 *
	 * @return
	 *
	 * @see
	 */
	public String getColumnName(int columnIndex) {
	    String name = new String();

	    switch (columnIndex) {

	    case 0:
		name = "Text to find";

		break;

	    case 1:
		name = "Replace with";

		break;
	    }

	    return name;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param columnIndex
	 *
	 * @return
	 *
	 * @see
	 */
	public Class getColumnClass(int columnIndex) {
	    Class  columnClass = null;
	    Object value = getValueAt(0, columnIndex);

	    if (value != null) {
		columnClass = value.getClass();
	    }

	    return columnClass;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param rowIndex
	 * @param columnIndex
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean isCellEditable(int rowIndex, int columnIndex) {
	    return false;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param rowIndex
	 * @param columnIndex
	 *
	 * @return
	 *
	 * @see
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
	    Object value = null;

	    if (!isTableEmpty()) {
		LocalRow  row = (LocalRow) rowVector.elementAt(rowIndex);
		LocalCell cell = new LocalCell(columnIndex, row);

		value = cell.getValue();
	    }

	    return value;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param aValue
	 * @param rowIndex
	 * @param columnIndex
	 *
	 * @see
	 */
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
	    if (!isTableEmpty()) {
		LocalRow  row = (LocalRow) rowVector.elementAt(rowIndex);
		LocalCell cell = new LocalCell(columnIndex, row);

		cell.setValue(aValue);
		fireTableCellUpdated(columnIndex, rowIndex);
	    }
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param rowIndex
	 *
	 * @return
	 *
	 * @see
	 */
	protected LocalRow getRow(int rowIndex) {
	    LocalRow row = null;

	    if (!isTableEmpty()) {
		if (rowVector.size() > rowIndex) {
		    row = (LocalRow) rowVector.elementAt(rowIndex);
		}
	    }

	    return row;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	protected Object[] getWideValues() {
	    Object[] wideValues = null;
	    String   stringValue = " ";
	    String   current = " ";
	    int      rowCount = 0;

	    rowCount = getRowCount();
	    wideValues = new Object[2];
	    wideValues[0] = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";
	    wideValues[1] = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";

	    for (int column = 1; column < wideValues.length; column++) {
		for (int row = 0; row < rowCount; row++) {
		    stringValue = (String) getValueAt(row, column);
		    stringValue = stringValue.trim();
		    current = (String) wideValues[column];

		    if (current == null) {
			wideValues[column] = stringValue;
		    } else {
			if (stringValue.trim().length() > current.length()) {
			    wideValues[column] = stringValue;
			}
		    }
		}
	    }

	    return wideValues;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @see
	 */
	protected void saveMap() {
            OtterProject project = node.getProject();
	    int	       rowCount = rowVector.size();
	    String[][] map = new String[rowCount][2];
	    LocalRow   row = null;

	    for (int i = 0; i < rowCount; i++) {
		row = (LocalRow) rowVector.elementAt(i);
		map[i][0] = row.getFind();
		map[i][1] = row.getReplaceWith();
	    }

	    project.putPackageMap(map);
	}

	/**
	 * Method declaration
	 *
	 *
	 * @see
	 */
	protected void populateModel() {
            OtterProject project = node.getProject();
	    String[][] map = project.getPackageMap();

	    if (map != null) {
		int rowCount = map.length;

		for (int i = 0; i < rowCount; i++) {
		    addRow(map[i][0], map[i][1]);
		}
	    }
	}

	// /
	// /

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	private boolean isTableEmpty() {
	    boolean empty = true;

	    if (rowVector != null) {
		if (rowVector.size() > 0) {
		    empty = false;
		}
	    }

	    return empty;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param f
	 * @param p
	 *
	 * @see
	 */
	private void addRow(String f, String p) {
	    LocalRow newRow = null;

	    newRow = new LocalRow(f, p);

	    rowVector.addElement(newRow);

	    int size = rowVector.size();

	    fireTableDataChanged();
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param index
	 *
	 * @see
	 */
	private void removeRow(int index) {
            if (index < rowVector.size()) {
  	      rowVector.removeElementAt(index);
	      fireTableDataChanged();
            }
	}

       private void removeAllRows() {
         rowVector.removeAllElements();
         fireTableDataChanged();
       }


    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class LocalRow {
	private String find = new String();
	private String replaceWith = new String();

	/**
	 * Constructor declaration
	 *
	 *
	 * @param f
	 * @param p
	 *
	 * @see
	 */
	public LocalRow(String f, String rw) {
	    find = f;
	    replaceWith = rw;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public String getFind() {
	    return find;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param f
	 *
	 * @see
	 */
	public void setFind(String f) {
	    find = f;

	    saveToProject();
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public String getReplaceWith() {
	    return replaceWith;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param p
	 *
	 * @see
	 */
	public void setReplaceWith(String rw) {
	    replaceWith = rw;

	    saveToProject();
	}

	/**
	 * Method declaration
	 *
	 *
	 * @see
	 */
	private void saveToProject() {}

    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class LocalCell {
	private LocalRow row;
	private int    column;

	/**
	 * Constructor declaration
	 *
	 *
	 * @param c
	 * @param r
	 *
	 * @see
	 */
	public LocalCell(int c, LocalRow r) {
	    column = c;
	    row = r;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param value
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean setValue(Object value) {
	    boolean set = true;

	    switch (column) {

	    case 0:
		row.setFind((String) value);

		break;

	    case 1:
		row.setReplaceWith((String) value);

		break;

	    default:
		set = false;

		break;
	    }

	    return set;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public Object getValue() {
	    Object value = null;

	    switch (column) {

	    case 0:
		value = row.getFind();

		break;

	    case 1:
		value = row.getReplaceWith();

		break;
	    }

	    return value;
	}

	// /
	// /

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	protected LocalRow getRow() {
	    return row;
	}

    }
}