
/**
 * Title:        JBuilder Tools for Enhydra<p>
 * Description:  JBuilder 3 Add in tools for working with Enhydra.<p>
 * Copyright:    Copyright (c) 1999<p>
 * Company:      Lutris<p>
 * @author Paul Mahar
 * @version
 */
package org.enhydra.ide.jbuilder.common.wizard.xmlc;

import org.enhydra.devsys.swing.ExtensionFilter;
import org.enhydra.ide.jbuilder.common.node.OtterNode;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;

public class OptionFilePanel extends JPanel {
  public OtterNode node;
  public JTextField textFile;
  public TitledBorder border;
  public GridBagLayout layoutMain = new GridBagLayout();
  public JButton buttonBrowse;
  public LocalButtonListener buttonListener;

  public OptionFilePanel() {
	try {
	    jbInit();
	    pmInit();
	} catch (Exception e) {
	    e.printStackTrace();
	}

  }

  private void pmInit() {
    buttonListener = new LocalButtonListener();
    buttonBrowse.addActionListener(buttonListener);
  }

  private void jbInit() throws Exception {
      buttonBrowse = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
      textFile = (JTextField) Beans.instantiate(getClass().getClassLoader(), JTextField.class.getName());

      border = BorderFactory.createTitledBorder("XMLC option file");
      textFile.setMinimumSize(new Dimension(100, 21));
    textFile.setPreferredSize(new Dimension(100, 21));
    textFile.setEditable(false);
      buttonBrowse.setText("Set...");

      this.setLayout(layoutMain);
      this.setBorder(border);
      this.add(textFile, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 100, 0));
      this.add(buttonBrowse, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));

  }

  public void setEnabled(boolean e) {
    super.setEnabled(e);
    buttonBrowse.setEnabled(e);
    textFile.setBackground(UIManager.getColor(e ? "TextField.background" : "Label.background"));
  }


  public OtterNode getNode() {
    return node;
  }

  public void setNode(OtterNode n) {
    String filename = null;
    node = n;
    if (node == null) {
      buttonBrowse.setEnabled(false);
    } else {
      buttonBrowse.setEnabled(true);
      filename = node.getOptionFilename();
    }
    if (filename != null) {
      textFile.setText(filename);
    }

  }

  public String getValidFilename() {
    String filename = textFile.getText().trim();
    if (! filename.toLowerCase().endsWith(".xmlc")) {
       filename = new String();
    } else {
      File f = new File(filename);
      if (! f.exists()) {
        filename = new String();
      }
    }
    if (filename.trim().length() == 0) {
      textFile.setText(new String());
    }
    return filename;
  }

  public boolean isValidFilename() {
    return (getValidFilename().length() > 0);
  }

    private void browseForFile() {
	JFileChooser    chooser;
	ExtensionFilter filter;
	File		file;
	String		fileDir = new String();
	String		fileName = new String();

	filter = new ExtensionFilter();

	filter.addExtension("xmlc");
	filter.setDescriptionTitle("XMLC option file");

	chooser = new JFileChooser();

	if (node != null) {
	    chooser.setCurrentDirectory(new File(node.getFilename()));
	}

	chooser.setFileFilter(filter);
	chooser.setDialogTitle("Select a XMLC option file");
	chooser.setApproveButtonText("OK");

	int v = chooser.showOpenDialog(this);

	this.requestFocus();
	buttonBrowse.requestFocus();

	if (v == JFileChooser.APPROVE_OPTION) {
           if (chooser.getSelectedFile() == null ||
            (! chooser.getSelectedFile().isFile())) {
	      textFile.setText(new String());
              node.setOptionFilename(new String());
           } else {
  	      fileDir = chooser.getCurrentDirectory().toString();
	      fileDir = stripSlash(fileDir);
	      fileName = chooser.getSelectedFile().getName();

	      textFile.setText(fileDir + File.separator + fileName);
              node.setOptionFilename(getValidFilename());
            }
	}

	chooser.removeAll();

	chooser = null;
    }

    private String stripSlash(String dir) {
	String stripped = new String(dir);

	if (dir != null) {
	    if (dir.length() > 0) {
		if (dir.endsWith(File.separator)) {
		    stripped = dir.substring(0, dir.length() - 1);
		}
	    }
	}

	return stripped;
    }

    private class LocalButtonListener implements ActionListener {
      public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source == buttonBrowse) {
          browseForFile();
        }
      }
    }


}