/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: XMLCPropertyPanel.java,v 1.1.2.4 2000/03/22 02:25:57 pmahar Exp $
 */

package org.enhydra.ide.jbuilder.common.wizard.xmlc;

import org.enhydra.ide.jbuilder.common.wizard.AbstractPropertyPanel;
import org.enhydra.devsys.swing.GridBagConstraintsX;
import org.enhydra.ide.jbuilder.common.node.OtterProject;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.beans.*;
import java.awt.event.*;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class XMLCPropertyPanel extends AbstractPropertyPanel {
    private boolean active = false;
    private LocalButtonListener buttonListener = null;
    private OtterProject	project = null;
    private GridBagLayout       layoutCompile;
    private JPanel		panelJavaSource;
    private GridBagLayout       layoutJavaSource;
    private JCheckBox		checkKeep;
    private JCheckBox		checkCompile;
    private TitledBorder	borderJavaSource;
    private TitledBorder	borderMappings;
    private JPanel		panelMappings;
    private GridBagLayout       layoutMap;
    private JButton		buttonMap;
    private JPanel		panelOutput;
    private GridBagLayout       layoutOutput;
    private JCheckBox		checkMapImages;
    private JCheckBox		checkParseInfo;
    private JCheckBox		checkPrintAccessMethods;
    private JCheckBox		checkPrintInfo;
    private JCheckBox		checkDump;
    private JCheckBox		checkVerbose;
    private JCheckBox		checkAddToProject;
    private JComboBox           comboMap;
    private JLabel              labelMap;
    private OptionFilePanel     optionFilePanel;

    private final String mapAll      = "All";
    private final String mapSelected = "Selected";
    private final String mapNone     = "None";
    private JPanel panelMake;
    private JCheckBox checkMake;
    private GridBagLayout layoutMake;
    private JTabbedPane tabPane;
    private JPanel panelCompile;
    private BorderLayout layoutMain;

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public XMLCPropertyPanel() {
	try {
	    jbInit();
	    pmInit();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterProject getProject() {
	return project;
    }

    /**
     * Method declaration
     *
     *
     * @param p
     *
     * @see
     */
    public void setProject(OtterProject p) {
	project = p;
    }

    public void readProperties() {
        optionFilePanel.setNode(project);
        if (comboMap.getItemCount() > 0) {
          comboMap.removeAllItems();
        }

	if (project.isCompileJavaAllowed()) {
           // Win32 and JDeveloper
            checkMake.setEnabled(false);
            checkMake.setSelected(false);
            comboMap.addItem(mapAll);
            comboMap.addItem(mapNone);
            if (project.getMapScope() == OtterProject.MAP_SCOPE_NONE) {
  	      comboMap.setSelectedItem(mapNone);
            } else {
    	      comboMap.setSelectedItem(mapAll);
              project.setMapScope(OtterProject.MAP_SCOPE_ALL);
            }

	    checkCompile.setEnabled(true);
	    checkCompile.setSelected(project.isCompileJavaSelected());
	    checkAddToProject.setEnabled(false);
	    checkAddToProject.setSelected(false);
	} else {
            // Primetime
            checkMake.setEnabled(true);
            checkMake.setSelected(project.isXMLCMake());
            comboMap.addItem(mapAll);
            comboMap.addItem(mapSelected);
            comboMap.addItem(mapNone);

            switch (project.getMapScope()) {
              case OtterProject.MAP_SCOPE_NONE:
   	           comboMap.setSelectedItem(mapNone);
                   break;
              case OtterProject.MAP_SCOPE_SELECTED:
 	           comboMap.setSelectedItem(mapSelected);
                   break;
              default:
                   project.setMapScope(OtterProject.MAP_SCOPE_ALL);
	           comboMap.setSelectedItem(mapAll);
                   break;
            }

	    checkCompile.setEnabled(false);
	    checkCompile.setSelected(false);
	    checkAddToProject.setEnabled(false);
	    checkAddToProject.setSelected(true);
	}
   	checkMapImages.setSelected(project.isMapImages());
   	checkDump.setSelected(project.isDumpSelected());
	checkParseInfo.setSelected(project.isParseInfoSelected());
	checkPrintAccessMethods.setSelected(project.isPrintAccessMethodsSelected());
	checkPrintInfo.setSelected(project.isPrintInfoSelected());
	checkVerbose.setSelected(project.isVerboseSelected());
        active = true;
    }

    public void writeProperties() {
        String item = (String)comboMap.getSelectedItem();
        item = item.trim();
        if (project != null && active) {
          if (item.equalsIgnoreCase(mapAll)) {
    	    project.setMapScope(OtterProject.MAP_SCOPE_ALL);
          } else if (item.equalsIgnoreCase(mapSelected)) {
    	    project.setMapScope(OtterProject.MAP_SCOPE_SELECTED);
          } else if (item.equalsIgnoreCase(mapNone)) {
  	    project.setMapScope(OtterProject.MAP_SCOPE_NONE);
          }
        }

	if (project.isCompileJavaAllowed()) {
           // Win32 and JDeveloper
	    project.setCompileJavaSelected(checkCompile.isSelected());
	} else {
            // Primetime
            project.setXMLCMake(checkMake.isSelected());
	}

   	project.setMapImages(checkMapImages.isSelected());
   	project.setDumpSelected(checkDump.isSelected());
	project.setParseInfoSelected(checkParseInfo.isSelected());
	project.setPrintAccessMethodsSelected(checkPrintAccessMethods.isSelected());
	project.setPrintInfoSelected(checkPrintInfo.isSelected());
	project.setVerboseSelected(checkVerbose.isSelected());
    }


    /**
     * Method declaration
     *
     *
     * @see
     */
    private void pmInit() {
        checkKeep.setSelected(true);
	checkKeep.setEnabled(false);
	buttonListener = new LocalButtonListener();
	buttonMap.addActionListener(buttonListener);
    }

    /**
     * Method declaration
     *
     *
     * @throws Exception
     *
     * @see
     */
    private void jbInit() throws Exception {
	layoutCompile =
	    (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
					      GridBagLayout.class.getName());
        optionFilePanel = (OptionFilePanel) Beans.instantiate(getClass().getClassLoader(), OptionFilePanel.class.getName());

	panelJavaSource =
	    (JPanel) Beans.instantiate(getClass().getClassLoader(),
				       JPanel.class.getName());
	layoutJavaSource =
	    (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
					      GridBagLayout.class.getName());
	checkMapImages = (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
						  JCheckBox.class.getName());

	checkKeep = (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
						  JCheckBox.class.getName());
	checkCompile =
	    (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
					  JCheckBox.class.getName());
	borderJavaSource = new TitledBorder("");
	borderMappings = new TitledBorder("");
	panelMappings =
	    (JPanel) Beans.instantiate(getClass().getClassLoader(),
				       JPanel.class.getName());
	layoutMap =
	    (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
					      GridBagLayout.class.getName());
	buttonMap =
	    (JButton) Beans.instantiate(getClass().getClassLoader(),
					JButton.class.getName());
	panelOutput = (JPanel) Beans.instantiate(getClass().getClassLoader(),
						JPanel.class.getName());
	layoutOutput =
	    (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
					      GridBagLayout.class.getName());
	checkParseInfo =
	    (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
					  JCheckBox.class.getName());
	checkPrintAccessMethods =
	    (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
					  JCheckBox.class.getName());
	checkPrintInfo =
	    (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
					  JCheckBox.class.getName());
	checkDump = (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
						  JCheckBox.class.getName());
	checkVerbose =
	    (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
					  JCheckBox.class.getName());
	checkAddToProject =
	    (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
					  JCheckBox.class.getName());

	comboMap = (JComboBox) Beans.instantiate(getClass().getClassLoader(), JComboBox.class.getName());
    labelMap = (JLabel) Beans.instantiate(getClass().getClassLoader(), JLabel.class.getName());
    panelMake = (JPanel) Beans.instantiate(getClass().getClassLoader(), JPanel.class.getName());
    checkMake = (JCheckBox) Beans.instantiate(getClass().getClassLoader(), JCheckBox.class.getName());
    layoutMake = (GridBagLayout) Beans.instantiate(getClass().getClassLoader(), GridBagLayout.class.getName());
    tabPane = (JTabbedPane) Beans.instantiate(getClass().getClassLoader(), JTabbedPane.class.getName());
    panelCompile = (JPanel) Beans.instantiate(getClass().getClassLoader(), JPanel.class.getName());
        layoutMain = (BorderLayout) Beans.instantiate(getClass().getClassLoader(), BorderLayout.class.getName());
        borderJavaSource.setTitle("Generated Java Source");
	borderMappings.setTitle("Mappings");
	panelJavaSource.setLayout(layoutJavaSource);
	panelJavaSource.setBorder(borderJavaSource);
	panelMappings.setBorder(borderMappings);
	panelOutput.setLayout(layoutOutput);
	checkKeep.setSelected(true);
	checkKeep.setText("Keep");
	checkCompile.setSelected(true);
	checkCompile.setText("Compile");
	panelMappings.setLayout(layoutMap);
	buttonMap.setText("Edit...");
	checkParseInfo.setText("Parsing details");
	checkParseInfo.setActionCommand("");
	checkPrintInfo.setText("IDs and URLs");
	checkPrintInfo.setActionCommand("Show information about the page object, such as ids and URLs");
	checkDump.setText("DOM tree");
	checkDump.setActionCommand("");
	checkVerbose.setActionCommand("");
	checkVerbose.setText("Compile process");
	checkPrintAccessMethods.setText("Access methods");
	checkAddToProject.setText("Add to Project");
	checkAddToProject.setActionCommand("");
        checkMapImages.setText("Use map to copy images into output directory");
	labelMap.setHorizontalAlignment(SwingConstants.RIGHT);
        labelMap.setText("Map Nodes:");
    checkMake.setEnabled(false);
    checkMake.setText("Invoke XMLC during Project Make/Rebuild");

    panelMake.setLayout(layoutMake);
    panelMake.add(checkMake, new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));

	panelJavaSource.add(checkKeep,
			    new GridBagConstraintsX(0, 0, 1, 1, 0.0, 0.0,
						   GridBagConstraints.CENTER,
						   GridBagConstraints.HORIZONTAL,
						   new Insets(5, 5, 5, 5), 5,
						   0));
	panelJavaSource.add(checkAddToProject,
			    new GridBagConstraintsX(1, 0, 1, 1, 0.0, 0.0,
						   GridBagConstraints.CENTER,
						   GridBagConstraints.HORIZONTAL,
						   new Insets(5, 5, 5, 5), 5,
						   0));
	panelJavaSource.add(checkCompile,
			    new GridBagConstraintsX(2, 0, 1, 1, 0.0, 0.0,
						   GridBagConstraints.CENTER,
						   GridBagConstraints.HORIZONTAL,
						   new Insets(5, 5, 5, 5), 5,
						   0));
	panelMappings.add(buttonMap, new GridBagConstraintsX(2, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 2, 5), 0, 0));
    panelMappings.add(comboMap, new GridBagConstraintsX(1, 0, 1, 1, 0.2, 0.2
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 2, 5), 0, 0));
    panelMappings.add(labelMap, new GridBagConstraintsX(0, 0, 1, 1, 0.2, 0.2
            ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 2, 5), 0, 0));
    panelMappings.add(checkMapImages, new GridBagConstraintsX(0, 1, 3, 1, 0.2, 0.2
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2, 5, 5, 5), 0, 0));


    panelCompile.setLayout(layoutCompile);
    panelCompile.add(panelJavaSource, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 5, 5));
    panelCompile.add(panelMappings, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 0), 5, 5));
    panelCompile.add(optionFilePanel, new GridBagConstraints(0, 3, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 5, 5, 5), 0, 0));


	panelOutput.add(checkPrintInfo,
		       new GridBagConstraintsX(1, 0, 1, 1, 0.0, 0.0,
					      GridBagConstraints.WEST,
					      GridBagConstraints.HORIZONTAL,
					      new Insets(0, 0, 0, 0), 0, 0));
	panelOutput.add(checkParseInfo,
		       new GridBagConstraintsX(1, 1, 1, 1, 0.0, 0.0,
					      GridBagConstraints.WEST,
					      GridBagConstraints.HORIZONTAL,
					      new Insets(5, 5, 5, 5), 5, 0));
	panelOutput.add(checkDump,
		       new GridBagConstraintsX(0, 2, 1, 1, 0.0, 0.0,
					      GridBagConstraints.WEST,
					      GridBagConstraints.HORIZONTAL,
					      new Insets(5, 5, 5, 5), 5, 0));
	panelOutput.add(checkVerbose,
		       new GridBagConstraintsX(0, 1, 1, 1, 0.0, 0.0,
					      GridBagConstraints.CENTER,
					      GridBagConstraints.NONE,
					      new Insets(5, 5, 5, 5), 5, 0));
	panelOutput.add(checkPrintAccessMethods,
		       new GridBagConstraintsX(0, 0, 1, 1, 0.0, 0.0,
					      GridBagConstraints.WEST,
					      GridBagConstraints.HORIZONTAL,
					      new Insets(5, 5, 5, 5), 5, 0));

    tabPane.add(panelCompile, "Make");
    tabPane.add(panelOutput, "Output");



    this.setLayout(layoutMain);
    this.add(tabPane, BorderLayout.CENTER);
    this.add(panelMake, BorderLayout.NORTH);
   }


    /**
     * Method declaration
     *
     *
     * @see
     */
    private void editProjectMap() {
	MapDialog dialog =
	    new MapDialog(((JDialog) this.getTopLevelAncestor()),
			  "Edit Project Map", true);

	dialog.setProject(project);
	dialog.show();
    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class LocalButtonListener implements ActionListener {

	/**
	 * Method declaration
	 *
	 *
	 * @param event
	 *
	 * @see
	 */
	public void actionPerformed(ActionEvent event) {
	    Object source = event.getSource();

	    if (source == buttonMap) {
		editProjectMap();
	    }
	}

    }


}
