/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: OracleProject.java,v 1.1.4.2 2000/03/22 02:25:57 pmahar Exp $
 */

package org.enhydra.ide.jbuilder.jdeveloper.node;

import java.util.Vector;
import java.io.File;
import borland.jbuilder.addin.Container;
import borland.jbuilder.addin.Node;
import borland.jbuilder.addin.Project;
import borland.jbuilder.addin.Url;
import org.enhydra.ide.jbuilder.common.node.OtterImageFileNode;
import org.enhydra.ide.jbuilder.common.node.OtterProject;
import org.enhydra.ide.jbuilder.common.node.OtterNode;
import org.enhydra.ide.jbuilder.common.node.OtterNodeFactory;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class OracleProject extends OtterProject implements OtterNode {
    private Project	   nativeProject = null;
    private OracleNodeFactory factory = null;

    public String	   xSourcePath;

    public OtterImageFileNode[] getAllImages() {
      ImageNodeReader reader = new ImageNodeReader();
      return reader.getNodes(nativeProject);
    }


    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getName() {
	String name = null;

	name = nativeProject.getName();

	if (name.startsWith(nativeProject.getPath())) {
	    name = name.substring(nativeProject.getPath().length() + 1);
	}

	if (name.indexOf(".") > 0) {
	    name = name.substring(0, name.indexOf("."));
	}

	return name;
    }

    /**
     * Constructor declaration
     *
     *
     * @param p
     *
     * @see
     */
    public OracleProject(Project p) {
	setNativeProject(p);

	factory = new OracleNodeFactory();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterNodeFactory getNodeFactory() {
	return factory;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public Object getNativeProject() {
	return nativeProject;
    }

    public OtterProject getProject() {
      return this;
    }

    /**
     * Method declaration
     *
     *
     * @param p
     *
     * @see
     */
    public void setNativeProject(Object p) {
	nativeProject = (Project) p;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public Object getNativeNode() {
	return getNativeProject();
    }

    /**
     * Method declaration
     *
     *
     * @param n
     *
     * @see
     */
    public void setNativeNode(Object n) {
	setNativeProject(n);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isCompileJavaAllowed() {
       return true;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getOutputDirectory() {
	String out = null;

	if (nativeProject != null) {
	    out = nativeProject.getOutputDirectory();
	}

	return out;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getSourcePath() {
	String out = null;

	if (nativeProject != null) {
	    out = nativeProject.getSourcePath();
	}

	return out;
    }

    /**
     * Method declaration
     *
     *
     * @param pathAsArray
     *
     * @see
     */
    public void setSourcePathArray(String[] pathAsArray) {
	StringBuffer path = new StringBuffer();

	for (int i = 0; i < pathAsArray.length; i++) {
	    path.append(pathAsArray[i] + ";");
	}

	nativeProject.setSourcePath(path.toString());
	setProperty("SourcePath", path.toString());

	xSourcePath = path.toString();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String[] getSourcePathArray() {
	String[] pathAsArray = null;
	String   pathAsString = getSourcePath();
	Vector   pathAsVector = new Vector();
	int      at = pathAsString.indexOf(";");
	int      size = 0;

	while (at > -1) {
	    pathAsVector.addElement(pathAsString.substring(0, at).trim());

	    pathAsString = pathAsString.substring(at + 1).trim();
	    at = pathAsString.indexOf(";");
	}

	pathAsVector.addElement(pathAsString.trim());

	size = pathAsVector.size();

	if (size > 0) {
	    pathAsArray = new String[size];

	    for (int i = 0; i < size; i++) {
		pathAsArray[i] = (String) pathAsVector.elementAt(i);
	    }
	}

	return pathAsArray;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getClassPath() {
	String out = null;

	if (nativeProject != null) {
	    out = nativeProject.getClassPath();
	}

	return out;
    }

    /**
     * Method declaration
     *
     *
     * @param property
     *
     * @return
     *
     * @see
     */
    public String getProperty(String property) {
	String out = null;

	if (nativeProject != null) {
	    out = nativeProject.getProperty(property);
	}

	return out;
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param value
     *
     * @see
     */
    public void setProperty(String property, String value) {
	if (nativeProject != null) {
	    nativeProject.setProperty(property, value);
	    nativeProject.save(true);
	}
    }

    public void setProperty(String property, int i) {
        String value = Integer.toString(i);
	setProperty(property, value);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getFilename() {
	return nativeProject.getUrl().getFilename();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public char getFileSeparatorChar() {
	return File.separatorChar;
    }

    public void configureRunClass() {
      System.out.println("configureRunClass not supported with this IDE");
    }

}

class ImageNodeReader {
  private Vector nodeVector = new Vector();

  protected ImageNodeReader() {
  }


  protected OtterImageFileNode[] getNodes(Project nativeProject) {
    OtterImageFileNode[] nodes;
    recursiveRead(nativeProject);
    nodes = new OtterImageFileNode[nodeVector.size()];
    for (int i = 0; i < nodes.length; i++) {
	nodes[i] = (OtterImageFileNode) nodeVector.elementAt(i);
    }
    return nodes;
  }


   private void recursiveRead(Container container) {
	String     filename = null;
	Node[]     nativeNodes = container.getChildren();
	File       current = null;

	for (int i = 0; i < nativeNodes.length; i++) {
	    if (nativeNodes[i] instanceof Container) {
		if (!nativeNodes[i].toString().equalsIgnoreCase("RootContainer")) {
		    recursiveRead((Container) nativeNodes[i]);
		}
	    } else {
		filename = nativeNodes[i].getFilename();
		current = new File(filename);
                if ((filename.toLowerCase().endsWith(".gif") ||
                     filename.toLowerCase().endsWith(".jpg")) &&
	             current.exists()) {
                    OtterImageFileNode imageNode = null;
                    imageNode = new OracleImageFileNode(nativeNodes[i]);
		    nodeVector.addElement(imageNode);
		}
	    }
	}
    }
}