/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: XMLCWizard.java,v 1.1.4.2 2000/03/22 02:25:58 pmahar Exp $
 */

package org.enhydra.ide.jbuilder.jdeveloper.wizard.xmlc;

import org.enhydra.devsys.swing.IconSet;
import org.enhydra.devsys.swing.SwingUtil;
import org.enhydra.xml.xmlc.XMLCException;
import org.enhydra.ide.jbuilder.common.node.OtterXMLCNode;
import org.enhydra.ide.jbuilder.common.wizard.xmlc.XMLCBuildDialog;
import org.enhydra.ide.jbuilder.jdeveloper.node.OracleProject;
import org.enhydra.ide.jbuilder.jdeveloper.node.OracleHTMLFileNode;
import borland.jbuilder.addin.Container;
import borland.jbuilder.addin.Context;
import borland.jbuilder.addin.IdeEnvironment;
import borland.jbuilder.addin.VersionedAddin;
import borland.jbuilder.addin.Wizard;
import borland.jbuilder.addin.WizardManager;
import borland.jbuilder.addin.Node;
import borland.jbuilder.addin.HtmlNode;
import borland.jbuilder.addin.Project;
import javax.swing.JOptionPane;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Image;
import java.io.File;
import java.io.Serializable;
import java.util.Vector;
import java.lang.System;

/**
 * This is the class that gets registered with the IDE through the
 * jbuilder.properties file.
 */
public class XMLCWizard implements Serializable, VersionedAddin, Wizard {
    private String       title = null;
    private Vector       sourceVector = new Vector();
    private int		 option = 0;

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public XMLCWizard() {}

    /**
     * For the VersionAddin Interface.
     */
    public String version() {
	return "2.0";
    }

    /**
     * Entry point exposed by the Wizard interface for early initialization.
     * Registers this Wizard object with the IDE's WizardManager object
     * @$param ide Access to all AddIn interfaces
     */
    public void startup(IdeEnvironment ide) {
	WizardManager e = ide.getWizardManager();

	e.registerWizard(this);

	// ?4Paul: Are we calling the following just for fun?
	Component c = ide.getIdeFrame();
	Class     cl = getClass();
    }

    /**
     * Wizard interface implementation, run the wizard in response to this call.
     * @param ide Access to all AddIn interfaces
     * @param context Current node, project, browser
     */
    public void invoke(IdeEnvironment ide, Context context) {
	Project      nativeProject = null;
	OtterXMLCNode[] xmlcNodes = null;
	Frame	     frame = null;

	frame = ide.getIdeFrame();
	sourceVector = new Vector();
	nativeProject = context.getProject();

	if (nativeProject != null
		&& isEnhydraAvailable(ide, nativeProject.getClassPath())) {
	    nativeProject.save(false);    // save all file changes in project
	    readNodes(nativeProject, ide.getIdeRootDir() + "bin");

	    XMLCBuildDialog dlg = new XMLCBuildDialog(frame, getName(), true);

	    xmlcNodes = new OtterXMLCNode[sourceVector.size()];

	    for (int i = 0; i < xmlcNodes.length; i++) {
		xmlcNodes[i] = (OtterXMLCNode) sourceVector.elementAt(i);
	    }

	    OracleProject oracleProject = null;

	    oracleProject = new OracleProject(nativeProject);
	    dlg.setProject(oracleProject);
	    dlg.setNodes(xmlcNodes);


	    dlg.show();
	    frame.requestFocus();
	}
    }

    /**
     * Wizard interface method, returns information about the type of Wizard this is.
     * @return An integer representing combined flags defined by the
     * Wizard interface.
     */
    public int getInfo() {
	return Wizard.PRODUCES_FILE + Wizard.REQUIRES_PROJECT;
    }

    /**
     * Wizard interface method, returns the name of this wizard.
     * @return Name of the wizard
     */
    public String getName() {
	return "XMLC Compiler Wizard";
    }

    /**
     * Wizard interface method, returns the text for the menu label
     * @return Menu label text
     */
    public java.lang.String getMenuLabel() {
	return getName() + "...";
    }

    /**
     * Wizard interface method
     * @return Currently returns no image (null)
     */
    public Image getSmallImage() {
	return IconSet.getSmallIcon().getImage();
    }

    /**
     * Wizard interface method
     * @return Currently returns no image (null)
     */
    public Image getBigImage() {
	return IconSet.getLargeIcon().getImage();
    }

    // ?4Paul: Are we keeping entry through showDialog()?

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getTitle() {
	return this.title;
    }

    /**
     * Method declaration
     *
     *
     * @param newTitle
     *
     * @see
     */
    public void setTitle(String newTitle) {
	this.title = newTitle;
    }

    // read only properties

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public int getOption() {
	return option;
    }

    /**
     * Method declaration
     *
     *
     * @param newOption
     *
     * @see
     */
    private void setOption(int newOption) {
	option = newOption;
    }

    /**
     * Method declaration
     *
     *
     * @param owner
     *
     * @see
     */
    public void showDialog(Component owner) {
	XMLCBuildDialog dialog = null;

	if (owner instanceof Dialog) {
	    dialog = new XMLCBuildDialog((Dialog) owner, getTitle(), true);
	} else if (owner instanceof Frame) {
	    dialog = new XMLCBuildDialog((Frame) owner, getTitle(), true);
	}

	dialog.show();
	dialog.clearAll();

	if (owner.isVisible()) {
	    owner.requestFocus();
	}
    }

    /**
     * Method declaration
     *
     *
     * @param args
     *
     * @see
     */
    public static void main(String[] args) {
	SwingUtil.setLookAndFeelToSystem();

	XMLCWizard wizard;

	wizard = new XMLCWizard();

	wizard.showDialog(new Frame());
	System.exit(0);
    }

    /**
     * Method declaration
     *
     *
     * @param ide
     * @param projectClassPath
     *
     * @return
     *
     * @see
     */
    private boolean isEnhydraAvailable(IdeEnvironment ide,
				       String projectClassPath) {
	String       message = null;

	try {
	    Class testClass = Class.forName(XMLCBuildDialog.XMLC_CLASS);
	} catch (java.lang.ClassNotFoundException e) {
	    message =
		"Enhydra 2.3 classes not found. \n"
		+ "Please verify that that the "
		+ "enhydra.jar file is included the Djava.class.path setting of "
		+ "the JBUILDER.INI file.\n\n";

	    e.printStackTrace();
	}

	if (message == null) {
	    if (projectClassPath.toLowerCase().indexOf("enhydra.jar") < 0) {
		message =
		    "The enhydra.jar file is not included in the "
		    + "current project library set.\n"
		    + "Select the Project | Properties menu "
		    + "to add it before using the Enhydra HTML Compiler.\n\n";
	    }
	}

	if (message != null) {
	    JOptionPane.showMessageDialog(null, message);
	}

	return (message == null);
    }

    // recursively read through nodes.

    /**
     * Method declaration
     *
     *
     * @param container
     *
     * @see
     */
    private void readNodes(Container container, String javacDir) {
	String     filename = null;
	Node[]     nativeNodes = container.getChildren();
	File       current = null;
	String     projectPage = new String();

	if (container instanceof Project) {
	    projectPage = container.getFilename();
	} else {
	    projectPage = container.getProject().getFilename();
	}

	if (projectPage.length() > 5) {
	    projectPage = projectPage.substring(0, projectPage.length() - 4);
	}

	projectPage += ".html";

	for (int i = 0; i < nativeNodes.length; i++) {
	    if (nativeNodes[i] instanceof Container) {
		if (!nativeNodes[i].toString().equalsIgnoreCase("RootContainer")) {
		    readNodes((Container) nativeNodes[i], javacDir);    // recrusive folder read.
		}
	    } else {
		filename = nativeNodes[i].getFilename();
		current = new File(filename);
                if ((nativeNodes[i] instanceof HtmlNode) &&
   		    (filename.toLowerCase().endsWith(".html")) &&
	            current.exists()) {
                    OracleHTMLFileNode oracleNode = null;
                    OtterXMLCNode  xmlcNode = null;
                    oracleNode  = new OracleHTMLFileNode((HtmlNode)nativeNodes[i]);
                    oracleNode.setJavacDirectory(javacDir);

                    try {
                      xmlcNode = new OtterXMLCNode(oracleNode);
  		      sourceVector.addElement(xmlcNode);
                    } catch (XMLCException e) {
                      e.printStackTrace();
                    }
		}
	    }
	}
    }
}
