/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: ConfBuilder.java,v 1.1.2.2 2000/03/21 09:47:47 pmahar Exp $
 */
package org.enhydra.ide.jbuilder.primetime.build;

import org.enhydra.ide.jbuilder.primetime.node.PrimeConfFileNode;
import org.enhydra.ide.jbuilder.common.node.OtterConfFileNode;
import org.enhydra.ide.jbuilder.primetime.properties.ConfProperties;
import org.enhydra.ide.jbuilder.common.PropUtil;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.node.FileNode;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.Project;
import com.borland.primetime.build.Builder;
import com.borland.primetime.build.BuildProcess;
import com.borland.primetime.build.BuilderManager;

import java.util.List;

/**
 * This class controls building and maintaining properties for HTML/XML files
 * using the Enhydra XMLC Compiler.
 *
 */
public class ConfBuilder implements Builder {
    public static final String GENERATE_CLASS = "Generate";

    public ConfBuilder() {}

    /**
     * Register open tool
     *
     */
    public static void initOpenTool(byte majorVersion, byte minorVersion) {
	BuilderManager.registerBuilder(new ConfBuilder());
    }

    /**
     * Called by the Build System when make or rebuild occurs
     */
    public void updateBuildProcess(BuildProcess buildProcess,
				   Node nativeNode) {
        ConfBuildTask     task      = null;
        List              taskList  = null;
        FileNode          fileNode  = null;
	OtterConfFileNode confNode  = null;
        String            stringIn  = null;
        boolean           selected  = false;

	if (nativeNode instanceof FileNode) {
	  fileNode = (FileNode) nativeNode;
	}

	if (fileNode != null &&
           fileNode.getUrl().getFileObject().getAbsolutePath().toLowerCase().endsWith("conf.in")) {
           confNode = new PrimeConfFileNode((TextFileNode)fileNode);
        }

        if (confNode != null && confNode.isSelected()) {
  	  task = (ConfBuildTask) buildProcess.getTask(ConfBuildTask.SHARED_KEY);
	  if (task == null) {
	      task = new ConfBuildTask();
	      // There is both a map and a list
	      buildProcess.putTask(ConfBuildTask.SHARED_KEY, task);
              taskList = buildProcess.getBuildTaskList();
              taskList.add(task);
	  }
 	  task.addNode(confNode);
        }
    }


}





