/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: XMLCBuilder.java,v 1.2.2.3 2000/03/22 02:25:58 pmahar Exp $
 */
package org.enhydra.ide.jbuilder.primetime.build;

import org.enhydra.ide.jbuilder.primetime.node.PrimeHTMLFileNode;
import org.enhydra.ide.jbuilder.common.node.OtterXMLCNode;
import org.enhydra.xml.xmlc.XMLCException;
import com.borland.jbuilder.node.HTMLFileNode;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.Project;
import com.borland.primetime.build.Builder;
import com.borland.primetime.build.BuildProcess;
import com.borland.primetime.build.BuilderManager;

import java.util.List;

/**
 * This class controls building and maintaining properties for HTML/XML files
 * using the Enhydra XMLC Compiler.
 *
 */
public class XMLCBuilder implements Builder {
    public static final String GENERATE_CLASS = "Generate";

    public XMLCBuilder() {}

    /**
     * Register open tool
     *
     */
    public static void initOpenTool(byte majorVersion, byte minorVersion) {
	BuilderManager.registerBuilder(new XMLCBuilder());
    }

    /**
     * Called by the Build System when make or rebuild occurs
     */
    public void updateBuildProcess(BuildProcess buildProcess,
				   Node nativeNode) {

	// TODO: ?? Do we want to support XMLFileNode ??
	PrimeHTMLFileNode primeNode = null;
	HTMLFileNode      htmlNode = null;
	OtterXMLCNode     xmlcNode = null;

	if (nativeNode instanceof HTMLFileNode) {
	    htmlNode = (HTMLFileNode) nativeNode;
	    primeNode = new PrimeHTMLFileNode(htmlNode);
            if (primeNode.getProject().isXMLCMake()) {

  	      // Check if we want to build this specific HTML file.
	      if (primeNode.isSelected()) {
		  XMLCBuildTask task =
		      (XMLCBuildTask) buildProcess.getTask(XMLCBuildTask.SHARED_KEY);

  		  if (task == null) {
		      task = new XMLCBuildTask();

		      // There is both a map and a list
		      buildProcess.putTask(XMLCBuildTask.SHARED_KEY, task);
                      // Add xmlc task to top of list so genrated DOM
                      // classes are available when dependent classes
                      // are compiled.
                      List list = buildProcess.getBuildTaskList();
                      list.add(0,task);

		  }

		  try {
		     xmlcNode = new OtterXMLCNode(primeNode);
    		     task.addNode(xmlcNode);
		  } catch (XMLCException e) {
		      e.printStackTrace();
		  }
               }
	    }
	}
    }


}





