/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: PrimeConfFileNode.java,v 1.1.2.1 2000/03/21 09:53:34 pmahar Exp $
 */

package org.enhydra.ide.jbuilder.primetime.node;

import org.enhydra.ide.jbuilder.common.PropUtil;
import org.enhydra.ide.jbuilder.common.node.OtterProject;
import org.enhydra.ide.jbuilder.common.node.OtterNode;
import org.enhydra.ide.jbuilder.common.node.OtterConfFileNode;
import org.enhydra.ide.jbuilder.common.node.OtterSelectionNode;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.node.Project;
import com.borland.primetime.node.Node;
import com.borland.primetime.vfs.Url;
import com.borland.primetime.node.DuplicateNodeException;
import java.io.File;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class PrimeConfFileNode extends PrimeNode
    implements OtterConfFileNode {


    private Exception exception;

    public PrimeConfFileNode(TextFileNode n) {
        super(n);
    }

    /**
     * Constructor declaration
     *
     *
     * @param otterProject
     * @param otterNode
     * @param file
     *
     * @see
     */
    public PrimeConfFileNode(OtterProject otterProject, OtterNode otterNode,
		         String file) {
	Project  nativeProject = null;
	Node	 nativeNode = null;
	TextFileNode nativeTextFile = null;

	if (otterProject.getNativeProject() instanceof Project) {
	    nativeProject = (Project) otterProject.getNativeProject();
	}

	if (otterNode.getNativeNode() instanceof Node) {
	    nativeNode = (Node) otterNode.getNativeNode();
	}

	if (nativeProject != null && nativeNode != null) {
	    try {
		nativeTextFile = new TextFileNode(nativeProject, nativeNode,
					  new Url(new File(file)));
	    } catch (DuplicateNodeException e) {
		System.err.println(e);
		System.err.println(file + " already in project.");
	    }
	}

	setNativeNode(nativeTextFile);
    }


  public boolean isSelected() {
    String in = getProperty(OtterSelectionNode.SELECTED);
    return PropUtil.stringToBoolean(in, false);
  }

  public void setSelected(boolean b) {
   setProperty(OtterSelectionNode.SELECTED, PropUtil.booleanToString(b));
  }


    /**
     * Save Exception information for reporting. This lets exceptions from
     * multiple nodes to be reported in a batch.
     *
     * @param e
     * An exception that occured while invoking XMLC.
     */
    public void setException(Exception e) {
	exception = e;
    }

    /**
     * Get an exception that occured when invoking XMLC. Null if the node
     * has not been compiled or was compiled without error.
     */
    public Exception getException() {
	return exception;
    }


  public File getOutputFileObject() {
    File out = new File(getFilename());
    String absolute = getProject().getConfOutputDirectory() +
        File.separator + out.getName();
    absolute = absolute.substring(0,absolute.length()-3);
    out = new File(absolute);
    return out;
  }


}
