/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: ConfProperties.java,v 1.1.2.3 2000/03/20 08:31:35 pmahar Exp $
 */

package org.enhydra.ide.jbuilder.primetime.properties;

import org.enhydra.ide.jbuilder.primetime.node.PrimeHTMLFileNode;
import org.enhydra.ide.jbuilder.common.node.OtterXMLCNode;
import org.enhydra.xml.xmlc.XMLCException;
import com.borland.primetime.node.FileNode;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.Project;
import com.borland.primetime.properties.PropertyPageFactory;
import com.borland.primetime.properties.PropertyPage;
import com.borland.primetime.properties.PropertyManager;
import com.borland.primetime.properties.PropertyGroup;

public class ConfProperties implements PropertyGroup {
  public static final String GENERATE_CLASS = "Generate";
  
  public ConfProperties() {
  }


  /**
   * Initialize this OpenTool
   */
  public static void initOpenTool(byte majorVersion, byte minorVersion) {
	PropertyManager.registerPropertyGroup(new ConfProperties());
  }

  /**
   * Returns a PropertyPageFactory for the project or for an
   * "conf.in" file node.  Null if topic is something else.
   */
  public PropertyPageFactory getPageFactory(final Object topic) {
    PropertyPageFactory factory = null;
    FileNode fn;
    Project project;
    if (topic instanceof FileNode) {
      fn = (FileNode) topic;
      project = ((Node)topic).getProject();
      if (fn.getUrl().getFileObject().getAbsolutePath().toLowerCase().endsWith("conf.in")
        && project != null) {
        // Node property page to set your XMLC properties
        factory = (new PropertyPageFactory("Enhydra Configure") {
          public PropertyPage createPropertyPage() {
            return new ConfNodePropertyPage(topic);
          }
        });
      }
    } else if (topic instanceof Project) {
        // Project property page to set your XMLC properties
        factory = (new PropertyPageFactory("Enhydra Configure") {
          public PropertyPage createPropertyPage() {
            return new ConfProjectPropertyPage(topic);
          }
        });
    }
    return factory;
  }

  public void initializeProperties() {
  }
}