/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: XMLCNodePropertyPanel.java,v 1.4.2.4 2000/03/26 21:33:41 pmahar Exp $
 */
package org.enhydra.ide.jbuilder.primetime.properties;

import org.enhydra.ide.jbuilder.common.node.OtterXMLCNode;
import org.enhydra.ide.jbuilder.common.wizard.xmlc.MapDialog;
import org.enhydra.ide.jbuilder.common.wizard.xmlc.OptionFilePanel;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.File;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

public class XMLCNodePropertyPanel extends JPanel {
  private OtterXMLCNode node;
  private GridBagLayout layoutMain;
  private JCheckBox checkCompile;
  private JCheckBox checkKeep;
  private JTextField textClassName;
  private JPanel panelClassName;
  private GridBagLayout layoutClassName;
  private JRadioButton radioDefault;
  private JRadioButton radioCustom;
  private JRadioButton radioMap;
  private JButton buttonMap;
  private OptionFilePanel optionFilePanel;
  private TitledBorder borderClassName;
  private ButtonGroup radioGroup;
  private LocalButtonListener buttonListener;

  public XMLCNodePropertyPanel() {
    try {
      jbInit();
      pmInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public OtterXMLCNode getNode() {
    return node;
  }

  public void setNode(OtterXMLCNode n) {
    node = n;
  }

  private void pmInit() throws Exception {
    radioGroup = (ButtonGroup) Beans.instantiate(getClass().getClassLoader(), ButtonGroup.class.getName());

    radioGroup.add(radioDefault);
    radioGroup.add(radioCustom);
    radioGroup.add(radioMap);
    radioSelection();
    buttonListener = new LocalButtonListener();
    buttonMap.addActionListener(buttonListener);
  }

  private void jbInit() throws Exception {
    checkCompile = (JCheckBox) Beans.instantiate(getClass().getClassLoader(), JCheckBox.class.getName());
    checkKeep    = (JCheckBox) Beans.instantiate(getClass().getClassLoader(), JCheckBox.class.getName());
    textClassName = (JTextField) Beans.instantiate(getClass().getClassLoader(), JTextField.class.getName());

    layoutMain = (GridBagLayout) Beans.instantiate(getClass().getClassLoader(), GridBagLayout.class.getName());

    panelClassName = (JPanel) Beans.instantiate(getClass().getClassLoader(), JPanel.class.getName());
    layoutClassName = (GridBagLayout) Beans.instantiate(getClass().getClassLoader(), GridBagLayout.class.getName());
    radioDefault = (JRadioButton) Beans.instantiate(getClass().getClassLoader(), JRadioButton.class.getName());
    radioCustom = (JRadioButton) Beans.instantiate(getClass().getClassLoader(), JRadioButton.class.getName());
    radioMap = (JRadioButton) Beans.instantiate(getClass().getClassLoader(), JRadioButton.class.getName());
    buttonMap = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    optionFilePanel = (OptionFilePanel) Beans.instantiate(getClass().getClassLoader(), OptionFilePanel.class.getName());

    checkCompile.setText("Generate DOM class");
    checkCompile.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        enableUI(checkCompile.isSelected());
      }
    });

    checkKeep.setText("Keep generated Java source");
    optionFilePanel.setLayout(optionFilePanel.getLayout());
    panelClassName.setLayout(layoutClassName);
    borderClassName = BorderFactory.createTitledBorder("Generated class name");
    panelClassName.setBorder(borderClassName);
    radioDefault.setSelected(true);
    radioDefault.setText("Default");
    radioDefault.addChangeListener(new javax.swing.event.ChangeListener() {

      public void stateChanged(ChangeEvent e) {
        if (radioDefault.isSelected()) {
          radioSelection();
        }
      }
    });
    radioCustom.setText("Custom");
    radioCustom.addChangeListener(new javax.swing.event.ChangeListener() {

      public void stateChanged(ChangeEvent e) {
        if (radioCustom.isSelected()) {
          radioSelection();
        }
      }
    });
    radioMap.setText("Mapped");
    radioMap.addChangeListener(new javax.swing.event.ChangeListener() {

      public void stateChanged(ChangeEvent e) {
        if (radioMap.isSelected()) {
          radioSelection();
        }
      }
    });
    buttonMap.setText("Map Table...");

    panelClassName.add(textClassName, new GridBagConstraints(0, 1, 4, GridBagConstraints.REMAINDER, 1.0, 1.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
    panelClassName.add(radioCustom, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    panelClassName.add(radioMap, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    panelClassName.add(radioDefault, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    panelClassName.add(buttonMap, new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));


    this.setLayout(layoutMain);
    this.add(checkCompile, new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
    this.add(checkKeep, new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 20, 5, 5), 0, 0));
    this.add(panelClassName, new GridBagConstraints(0, 2, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 15, 3, 5), 0, 0));
    this.add(optionFilePanel, new GridBagConstraints(0, 3, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 15, 5, 5), 0, 0));
  }

  public void readProperties() {
    boolean b = false;
    String s = new String();;

    b = node.isSelected();
    checkCompile.setSelected(b);

    if (node.getClassNameType() == OtterXMLCNode.CLASS_NAME_CUSTOM) {
      radioCustom.setSelected(true);
    } else if (node.getClassNameType() == OtterXMLCNode.CLASS_NAME_MAPPED) {
      radioMap.setSelected(true);
    } else {
      radioDefault.setSelected(true);
    }
    s = node.getOptions().getClassName();
    textClassName.setText(s);

    checkKeep.setSelected(true);
    checkKeep.setEnabled(false);
    optionFilePanel.setNode(node);
    enableUI(checkCompile.isSelected());
    radioSelection();
  }

  public void writeProperties() {
    node.setSelected(checkCompile.isSelected());
    node.getOptions().setKeep(checkKeep.isSelected());

    String customClassName = textClassName.getText();
    node.setCustomClassName(customClassName);

    if (radioCustom.isSelected()) {
      node.setClassNameType(OtterXMLCNode.CLASS_NAME_CUSTOM);
    } else if (radioDefault.isSelected()) {
      node.setClassNameType(OtterXMLCNode.CLASS_NAME_DEFAULT);
    } else {
      node.setClassNameType(OtterXMLCNode.CLASS_NAME_MAPPED);
    }

    if (optionFilePanel.isValidFilename()) {
       node.setOptionFilename(optionFilePanel.getValidFilename());
    }
  }

  private void enableUI(boolean enable) {
    radioDefault.setEnabled(enable);
    radioCustom.setEnabled(enable);
    radioMap.setEnabled(enable);
    textClassName.setEnabled(radioCustom.isSelected());
    checkKeep.setEnabled(false);
    textClassName.setBackground(UIManager.getColor(textClassName.isEnabled() ? "TextField.background" : "Label.background"));
    buttonMap.setEnabled(enable);
    optionFilePanel.setEnabled(enable);
  }

  private void radioSelection() {
    textClassName.setEnabled(radioCustom.isSelected());
    textClassName.setBackground(UIManager.getColor(textClassName.isEnabled() ? "TextField.background" : "Label.background"));
    if (node != null) {
      writeProperties();
      String s = node.getOptions().getClassName();
      textClassName.setText(s);
    }
  }

     private void editMap() {
	MapDialog dialog =
	    new MapDialog(((JDialog) this.getTopLevelAncestor()),
			  "Edit Project Map", true);
	dialog.setProject(node.getProject());
	dialog.show();
    }



    private class LocalButtonListener implements ActionListener {
      public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source == buttonMap) {
          editMap();
        }
      }
    }

}