/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: AbstractBasicWizard.java,v 1.1.2.1 2000/03/20 08:38:40 pmahar Exp $
 */

package org.enhydra.ide.jbuilder.primetime.wizard;


import org.enhydra.devsys.swing.IconSet;
import org.enhydra.xml.xmlc.XMLCException;
import org.enhydra.ide.jbuilder.primetime.node.PrimeProject;
import org.enhydra.ide.jbuilder.primetime.node.PrimeHTMLFileNode;
import org.enhydra.ide.jbuilder.common.wizard.xmlc.XMLCBuildDialog;
import org.enhydra.ide.jbuilder.common.node.OtterXMLCNode;

import com.borland.jbuilder.node.JBProject;
import com.borland.jbuilder.node.HTMLFileNode;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.ide.BrowserListener;
import com.borland.primetime.ide.NodeViewer;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.Project;
import com.borland.primetime.wizard.BasicWizard;
import com.borland.primetime.wizard.Wizard;
import com.borland.primetime.wizard.WizardAction;
import com.borland.primetime.wizard.WizardHost;
import com.borland.primetime.wizard.WizardManager;
import com.borland.primetime.wizard.WizardPage;
import com.borland.primetime.util.VetoException;
import com.borland.primetime.vfs.InvalidUrlException;
import javax.swing.JOptionPane;
import java.io.File;
import java.util.Vector;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
abstract public class AbstractBasicWizard extends BasicWizard {
    private Vector	       sourceVector = new Vector();
    private JBProject	       nativeProject = null;
    public static boolean enableMenu = false;

    public static BrowserListener browserListener =
        new AbstractBasicWizardBrowserListener();

    public JBProject getNativeProject() {
      return nativeProject;
    }

    public void setNativeProject(JBProject p) {
      nativeProject = p;
    }

    public Vector getSourceVector() {
      return sourceVector;
    }
    /**
     * Method declaration
     *
     *
     * @param host
     *
     * @return
     *
     * @see
     */
    public WizardPage invokeWizard(WizardHost host) {
	setWizardTitle(getName());
        InvokerThread invoker = new InvokerThread(host);
        invoker.run();
	return super.invokeWizard(host);
    }

    /**
     * Method declaration
     *
     *
     * @throws VetoException
     *
     * @see
     */
    protected void finish() throws VetoException {}

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public static String getName() {
      return "Enhydra Wizard";
    }

    public static class AbstractBasicWizardBrowserListener implements BrowserListener {
      public void browserActivated(Browser browser) {}
      public void browserClosed(Browser browser) {}
      public void browserClosing(Browser browser) throws VetoException {}
      public void browserDeactivated(Browser browser) {}
      public void browserOpened(Browser browser) {}
      public void browserProjectActivated(Browser browser, Project project) {
        AbstractBasicWizard.enableMenu = (project != null);
      }
      public void browserProjectClosed(Browser browser, Project project) {}
      public void browserNodeActivated(Browser browser, Node node) {}
      public void browserNodeClosed(Browser browser, Node node) {}
      public void browserViewerActivated(Browser b, Node n, NodeViewer v) {}
      public void browserViewerDeactivating(Browser b, Node n, NodeViewer v)
           throws VetoException {}

    }


    /**
     * Method declaration
     *
     *
     * @param host
     *
     * @see
     */
    abstract public void invoke(WizardHost host);

    /**
     * Method declaration
     *
     *
     * @param projectClassPath
     *
     * @return
     *
     * @see
     */
    public boolean isEnhydraAvailable(String projectClassPath) {
	String message = null;

	try {
	    Class testClass = Class.forName(XMLCBuildDialog.XMLC_CLASS);
	} catch (java.lang.ClassNotFoundException e) {
	    message = "Enhydra not found in classpath. \n"
		      + "Please verify that that the "
		      + "enhydra.jar file is included your classpath.";
	}

	if (message == null) {
    	   if (!nativeProject.getPaths().putClassOnFullPath(XMLCBuildDialog.XMLC_CLASS)) {
		message =
		    "The enhydra.jar file is not included in the "
		    + "current project library set.\n"
		    + "Select the Project | Properties menu "
		    + "to add it before using the Enhydra HTML Compiler.\n\n";
	    }
	}

	if (message != null) {
	    JOptionPane.showMessageDialog(null, message);
	}

	return (message == null);
    }


    class InvokerThread extends Thread {
        private WizardHost host;

        public InvokerThread(WizardHost h) {
          host = h;
        }

        public void run() {
      	   invoke(host);
        }
    }

}
